#include <gtk/gtk.h>
#include <stdlib.h>
#include "define.h"
#include "../client_basic_modules/common/module.h"
#include "edit_layout.h"

void add_to_layout(mother)
struct mother_hold *mother;
{
	node_module_type *module_slider;
	int number_of_images;
	int count_down;
	node_layout_ptr new_element;
	node_layout_ptr layout_slider;
	int module_count;

	/**
	 * 'count_down' represents how many element images need to be
	 * gone through before the correct location is found.
	 *
	 * Finding the location is guaranteed.
	 */
	count_down = mother->server_form.tab2.current_edit_toggle;
	module_count = 0;
	module_slider = mother->module_head;
	while(module_slider) {
		number_of_images = module_slider->info->number_of_small_images;
		/* Check if the image is one of these which are available */
		if(count_down < number_of_images) {
			break;
		}
		count_down -= number_of_images;

		/* No? Skip to next module and check there... */
		module_slider = module_slider->next_module;
		++module_count;
	}

	/* Location found. */
/*
	g_print("We are wanting to add to location %d, instance %d, of %s\n",
			mother->server_form.tab2.current_display_toggle,
			count_down,
			module_slider->name);
*/

	/**
	 * Add this element to our design layout listing
	 */ 
	/* Allocate memory for the node */
	new_element = (node_layout_ptr) calloc (1, sizeof(node_layout_type));
	/* Fill in the new information about the module */
	new_element->id = module_count;
	new_element->x = mother->server_form.tab2.current_display_toggle %
		gtk_spin_button_get_value_as_int(
				GTK_SPIN_BUTTON(mother->server_form.tab2.spin_world_button_x));
	new_element->y = mother->server_form.tab2.current_display_toggle /
	    gtk_spin_button_get_value_as_int(
				GTK_SPIN_BUTTON(mother->server_form.tab2.spin_world_button_x));
	new_element->location = mother->server_form.tab2.current_display_toggle;
	new_element->instance = count_down;
	new_element->display_depth = get_depth_from_id(mother, new_element->id);
	new_element->next_element = NULL;

	/* Move to the end of the node list & attach */
	layout_slider = mother->design_head;
	if(layout_slider == NULL) {
		/* This is the first element */
		mother->design_head = new_element;
	}
	else if(layout_slider->next_element == NULL) {
		/* There is only one element, does this go before or after it? */
		if(layout_slider->display_depth >= new_element->display_depth) {
			/* place the new node before this one */
			new_element->next_element = mother->design_head;
			mother->design_head = new_element;
		}
		else {
			/* after this node */
			layout_slider->next_element = new_element;
		}
	}
	else {
		/* There are at least two elements, try to find the current location */
		/* and add appropriately according to depth drawing order. If */
		/* the location is unoccupied in the listing, add this */
		/* new node to the appropriate place in the list end. */
		layout_slider = mother->design_head;
		/* First case is special. */
		if(layout_slider->location == new_element->location) {
			/* If this drawing depth is less than the stored one then this */
			/* new node should be stored before it. There are 4 possible */
			/* depths so each must be checked for. */
			if(layout_slider->display_depth >= new_element->display_depth) {
				/* this new node is going to be the head node. */
				new_element->next_element = mother->design_head;
				mother->design_head = new_element;
			}
			else {
				while(layout_slider->next_element->location == new_element->location) {
					if(layout_slider->next_element->display_depth >= new_element->display_depth) {
						break;
					}
					else {
						layout_slider = layout_slider->next_element;
						if(layout_slider->next_element == NULL) {
							break;
						}
					}
				}
				/* Attach node here. */
				new_element->next_element = layout_slider->next_element;
				layout_slider->next_element = new_element;
			}
		}
		else {
			/* Find our element id in the node list */
			while(layout_slider->next_element->location != new_element->location) {
				layout_slider = layout_slider->next_element;
				if(layout_slider->next_element == NULL) {
					/* Special case, we have run off the end of the node list */
					layout_slider->next_element = new_element;
//	list_design_layout(mother);
					return;
				}
			}
			while(layout_slider->next_element->location == new_element->location) {
				if(layout_slider->next_element->display_depth >= new_element->display_depth) {
					break;
				}
				else {
					layout_slider = layout_slider->next_element;
					if(layout_slider->next_element == NULL) {
						break;
					}
				}
			}
			/* Attach node here. */
			new_element->next_element = layout_slider->next_element;
			layout_slider->next_element = new_element;
		}
	}

//	list_design_layout(mother);
}

void list_design_layout(mother)
struct mother_hold *mother;
{
	struct node_layout_type *layout_slider;
 
	layout_slider = mother->design_head;
	g_print("Listing design layout description.\n");
	while(layout_slider) {
		g_print("- Element Found of type: %d, at (%ld,%ld)(%ld), instance #%d, depth %d\n",
				layout_slider->id,
				layout_slider->x,
				layout_slider->y,
				layout_slider->location,
				layout_slider->instance,
				layout_slider->display_depth);
 
		/* move on to next element */
		layout_slider = layout_slider->next_element;
	}
	g_print("done.\n");
}                                                                                         

int get_depth_from_id(mother, id)
struct mother_hold *mother;
int id;
{
	struct node_module_type *module_slider;
	int module_ptr;

	module_ptr = 0;
	module_slider = mother->module_head;
	while(module_slider) {
		if(module_ptr == id) {
			break;
		}

		module_ptr++;
		module_slider = module_slider->next_module;
	}

	return module_slider->info->display_depth;
}
