#include <ncurses.h>
#include <panel.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "define.h"
#include "netutil.h"
#include "../server_basic_modules/common/recvenum.h"
#include "../server_basic_modules/common/sendenum.h"
#include "showhex.h"
#include "parsecli.h"
#include "make_world.h"
#include "../client_basic_modules/common/module.h"
#include "./art/p_card.xpm"
#include "netmod.h"

char* find_user(int, struct mother_hold *);
node_user_ptr find_node_user_ptr_from_id(int, struct mother_hold*);
void case_join(int, struct mother_hold*);
void case_talk(int, struct mother_hold*);
void case_emote(int, struct mother_hold*);
void case_whisper(int, struct mother_hold*);
void case_nick(int, struct mother_hold*);
void case_user_listing(int, struct mother_hold*);
void update_user_list(struct mother_hold *mother);
void case_error(int, struct mother_hold*);
void case_quit(int, struct mother_hold*);
void case_admin_access(int, struct mother_hold*);
void case_spawning_robot(int, struct mother_hold*);
void case_module_list(int, struct mother_hold*);
void case_element_placement(int, struct mother_hold*);
void case_world_size(int, struct mother_hold*);
void case_ending_layout_description(int, struct mother_hold*);
void case_program_card_list(int, struct mother_hold*);
void case_spawn_all_players(int, struct mother_hold*);
void case_initial_archive_location(int, struct mother_hold*);
void case_action(int, struct mother_hold*);
void case_request_spawning_direction(int, struct mother_hold*);
void case_game_is_now_starting(int, struct mother_hold*);
void case_game_is_now_stopping(int, struct mother_hold*);
void case_number_of_elements(int, struct mother_hold*);

struct node_module_type* find_module_by_server_ref_num(
		struct mother_hold*, int);

void parse_server(gpointer data, gint source, GdkInputCondition condition) {
	struct mother_hold *mother;
	char buff[512];
	char tmp[512];

	mother = (struct mother_hold*)data;

	/* Read off a single character, after that we'll know how many should
	 * be presant to be read and if they aren't then it is a malformed
	 * command sent by the client.
	 */
	readn(source, buff, 1);

	sprintf(tmp, "<com: #%d>", buff[0]);
	printf("%s\n", tmp);
/*
	sprintf(tmp, "\n<com: #%d>", buff[0]);
    gtk_text_insert(
	        GTK_TEXT(mother->main_form.chat_display),
	        NULL, NULL, NULL, tmp, strlen(tmp));
*/

	switch(buff[0]) {
		case S_QUIT:
			case_quit(source, mother);
			break;
		case S_JOIN:
			case_join(source, mother);
			break;
		case S_USER_LIST:
			case_user_listing(source, mother);
			break;
		case S_WHISPER:
			case_whisper(source, mother);
			break;
		case S_EMOTE: case_emote(source, mother);
			break;
		case S_TALK:
			case_talk(source, mother);
			break;
		case S_CHANGE_NICK:
			case_nick(source, mother);
			break;
		case S_ADMIN_ACCESS:
			case_admin_access(source, mother);
			break;
		case S_GAME_IS_NOW_STARTING:
			case_game_is_now_starting(source, mother);
			break;
		case S_GAME_IS_NOW_STOPPED:
			case_game_is_now_stopping(source, mother);
			break;
		case S_REQUEST_SPAWNING_DIRECTION:
			case_request_spawning_direction(source, mother);
			break;
		case S_REPLY_SPAWNING_DIRECTION:
			readn(source, tmp, 2);
			g_print("** Server SPAWNING-REPLY: user=%d, answer=%d\n",
					tmp[0], tmp[1]);
			if(!tmp[1]) {
				gtk_widget_show(mother->main_form.spawn_select_form);
			}
			break;
		case S_ELEMENT_LIST:
			case_module_list(source, mother);
			break;
		case S_STARTING_LAYOUT_DESCRIPTION:
			break;
		case S_WORLD_SIZE_IS:
			case_world_size(source, mother);
			break;
		case S_ELEMENT_PLACEMENT:
			case_element_placement(source, mother);
			break;
		case S_ENDING_LAYOUT_DESCRIPTION:
			case_ending_layout_description(source, mother);
			break;
		case S_INITIAL_ARCHIVE_LOCATION:
			case_initial_archive_location(source, mother);
			break;
		case S_PROGRAM_CARD_LIST:
			case_program_card_list(source, mother);
			break;
		case S_REPLY_PROGRAM_CARD:
			readn(source, buff, 2);
			g_print("Program card selection result of %d is: %d\n",
					buff[0], buff[1]);
			if(!buff[1]) {
				gtk_widget_show(mother->main_form.pcard_select_form);
			}
			break;
		case S_SPAWN_ALL_PLAYERS:
			case_spawn_all_players(source, mother);
			break;
		case S_ACTION:
			case_action(source, mother);
			break;
		case S_NUMBER_OF_ELEMENTS:
			case_number_of_elements(source, mother);
			break;
		default:
			g_print("Unknown data recieved: ");
			show_as_hex(1, buff);
			g_print("\n");
			g_print("decimal is %d\n", buff[0]);
			g_print("Closing down socket to avoid a crash...\n");
			close(mother->socket);
			mother->socket = 0;
	}
}

void case_number_of_elements(int source, struct mother_hold *mother) {
	char buff[512];
	int length;
	FILE *outfile;
	char *ptr;

	readn(source, buff, 1);
	length = buff[0];

	readn(source, buff, length);
	mother->number_of_elements = atol(buff);

	g_print("recieved that there are '%s' = %ld number of elements\n",
			buff, mother->number_of_elements);

	/* write this information out to the file that will be loaded */
	ptr = getenv("HOME");
	strcpy(buff, ptr);
	strcat(buff, "/.robot_race/world.brd");
	outfile = fopen(buff, "a");
	if(outfile == NULL) {
		g_print("Error: Unable to open '%s'.\n", buff);
		g_print("Error: Does the directory exist? Abandoning write.\n");
		return;
	}

	sprintf(buff, "%ld\n", mother->number_of_elements);
	fputs(buff, outfile);

	fclose(outfile);
}

void case_game_is_now_stopping(int source, struct mother_hold *mother) {
	struct node_layout_type *game_slider;
	struct node_layout_type *kill_me;

	g_print("** Game has been stopped.\n");

	gtk_widget_hide(mother->main_form.pcard_select_form);
	gtk_widget_hide(mother->main_form.spawn_select_form);

	game_slider = mother->game_head;
	while(game_slider) {
		kill_me = game_slider;
		game_slider = game_slider->next_element;

		g_free(kill_me);
	}
	mother->game_head = NULL;
}

void case_game_is_now_starting(int source, struct mother_hold *mother) {
	g_print("** Game has been started.\n");
}

void case_request_spawning_direction(int source, struct mother_hold *mother) {
	char buff[512];

	readn(source, buff, 1);
	g_print("** Server REQUEST: Send spawning direction %d\n",
			buff[0]);

	gtk_widget_show(mother->main_form.spawn_select_form);
}

void case_action(int source, struct mother_hold *mother) {
	struct node_module_type *module_slider;
	char buff[512];

	readn(source, buff, 3);

	g_print("ACTION (%d:%d)->(%d)\n",
			buff[0], buff[1], buff[2]);

	/**
	 * find the module associated with this network command and call
	 * the appropriate function inside it
	 */
	module_slider = find_module_by_server_ref_num(mother, buff[2]);

	/* call the network handling function in the module */
	call_network_module(mother, module_slider, buff);
}

void case_initial_archive_location(int source, struct mother_hold *mother) {
	char buff[512];

	g_print("The initial archive location is: %d,%d\n",
			buff[0], buff[1]);
	readn(source, buff, 2);

	mother->initial_archive_location_x = buff[0];
	mother->initial_archive_location_y = buff[1];
}

void case_spawn_all_players(int source, struct mother_hold *mother) {
	struct node_robot_type *robot_slider;
	struct node_robot_type *new_robot_ptr;
	char buff[512];
	int number_of_players, ctr;
	int is_virtual;

	g_print("At this time spawn all the players.\n");

	/* how many players are there */
	readn(source, buff, 1);
	number_of_players = buff[0];

	g_print("Listing %d player's initial facing direction.\n",
			number_of_players);

	if(number_of_players > 1) {
		is_virtual = TRUE;
	}
	else {
		is_virtual = FALSE;
	}

	mother->robot_head = NULL;
	for(ctr = 0; ctr < number_of_players; ctr ++) {
		readn(source, buff, 2);
		g_print("Player id#%d is initially facing %d\n", buff[0], buff[1]);

		/* Allocate a new robot node for this new user */
		new_robot_ptr = (node_robot_ptr) calloc (1, sizeof(node_robot_type));   
		new_robot_ptr->id = buff[0];
		new_robot_ptr->facing = buff[1];
		new_robot_ptr->is_virtual = is_virtual;
		new_robot_ptr->is_shutdown = FALSE;
		new_robot_ptr->current_x = mother->initial_archive_location_x;
		new_robot_ptr->current_y = mother->initial_archive_location_y;
		new_robot_ptr->archive_x = mother->initial_archive_location_x;
		new_robot_ptr->archive_y = mother->initial_archive_location_y;
		new_robot_ptr->next_robot = NULL;

		/* Attach this new node */
		if(mother->robot_head == NULL) {
			mother->robot_head = new_robot_ptr;
			robot_slider = mother->robot_head;
		}
		else {
			robot_slider->next_robot = new_robot_ptr;
			robot_slider = robot_slider->next_robot;
		}
	}
}

void case_program_card_list(int source, struct mother_hold *mother) {
	char buff[512], tmp[512];
	int number_of_cards, ctr;
	int curr_pcard;
	GdkColormap *colormap;
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;

	/**
	 * clear the selected cards to blanks
	 */
	colormap = gtk_widget_get_colormap(mother->main_form.pcard_select_form);
	pixmap = gdk_pixmap_colormap_create_from_xpm_d(
			NULL, colormap, &bitmap,
			NULL, p_card_xpm);

	for(ctr = 0; ctr < 5; ctr ++) {
		gtk_pixmap_set(GTK_PIXMAP(
				mother->main_form.select[ctr]),
				pixmap, bitmap);
		gtk_label_set_text(GTK_LABEL(
				mother->main_form.label_select[ctr]),
				"");
	}

	gdk_pixmap_unref(pixmap);

	/* how many cards are there */
	readn(source, buff, 1);
	number_of_cards = buff[0];

	/* read the cards off the network stream */
	readn(source, buff, number_of_cards);

	g_print("Program card listing: ");
	for(ctr = 0; ctr < number_of_cards; ctr ++) {
		curr_pcard = buff[ctr];

		if(curr_pcard <= 6) {
			/* u_turn */
			gtk_pixmap_set(GTK_PIXMAP(
					mother->main_form.pickme[ctr]),
					mother->main_form.program_cards_pixmap[0],
					mother->main_form.program_cards_bitmap[0]);

		}
		else if(curr_pcard <= 42) {
			if(!(curr_pcard % 2)) {
				/* rotate_right */
				gtk_pixmap_set(GTK_PIXMAP(
						mother->main_form.pickme[ctr]),
						mother->main_form.program_cards_pixmap[2],
						mother->main_form.program_cards_bitmap[2]);
			}
			else {
				/* rotate_left */
				gtk_pixmap_set(GTK_PIXMAP(
						mother->main_form.pickme[ctr]),
						mother->main_form.program_cards_pixmap[1],
						mother->main_form.program_cards_bitmap[1]);
			}
		}
		else if(curr_pcard <= 48) {
			/* back_up */
			gtk_pixmap_set(GTK_PIXMAP(
					mother->main_form.pickme[ctr]),
					mother->main_form.program_cards_pixmap[3],
					mother->main_form.program_cards_bitmap[3]);
		}
		else if(curr_pcard <= 66) {
			/* forward_1 */
			gtk_pixmap_set(GTK_PIXMAP(
					mother->main_form.pickme[ctr]),
					mother->main_form.program_cards_pixmap[4],
					mother->main_form.program_cards_bitmap[4]);
		}
		else if(curr_pcard <= 78) {
			/* forward_2 */
			gtk_pixmap_set(GTK_PIXMAP(
					mother->main_form.pickme[ctr]),
					mother->main_form.program_cards_pixmap[5],
					mother->main_form.program_cards_bitmap[5]);
		}
		else {
			/* forward_3 */
			gtk_pixmap_set(GTK_PIXMAP(
					mother->main_form.pickme[ctr]),
					mother->main_form.program_cards_pixmap[6],
					mother->main_form.program_cards_bitmap[6]);
		}
		sprintf(tmp, "%d", buff[ctr]);
		gtk_label_set_text(GTK_LABEL(
					mother->main_form.label_pickme[ctr]), tmp);

		g_print("%d,", buff[ctr]);
	}
	g_print("\n");

	gtk_widget_show(mother->main_form.pcard_select_form);
}

void case_ending_layout_description(int source, struct mother_hold *mother) {
	/* create and display this world we have just loaded */
	make_world(mother);
}

void case_world_size(int source, struct mother_hold *mother) {
	node_user_ptr server_slider;
	char buff[512], *ptr;
	FILE *outfile;
	int width, height;

	/* We want to store the board description in ~/.robot_race */
	/* under the filename: 'world.brd'. This will always be the */
	/* filename used for the layout being played on. */

	ptr = getenv("HOME");
	strcpy(buff, ptr);
	strcat(buff, "/.robot_race/world.brd");
	outfile = fopen(buff, "w");
	if(outfile == NULL) {
		g_print("Error: Unable to open '%s'.\n", buff);
		g_print("Error: Does the directory exist? Abandoning write.\n");
		return;
	}

	/* Read the size of the world from the network stream. */
	readn(source, buff, 2);
//	printf("The size of the world is: (%d, %d)\n",
//			buff[0], buff[1]);

	width = buff[0];
	height = buff[1];

	mother->main_form.width = width;
	mother->main_form.height = height;
				 
	/* Write the size of the board to our file. */
	sprintf(buff, "%d %d\n", width, height);
	fputs(buff, outfile);

	/* Write out the server servers */
	server_slider = mother->server_head;
	while(server_slider) {
		sprintf(buff, "module: %s, %s, %d\n", 
				server_slider->nick, "?.??", server_slider->id);
		fputs(buff, outfile);

		/* move to next element */
		server_slider = server_slider->next_user;
	}
	fputs("listing start:\n", outfile);

	fclose(outfile);
}

void case_element_placement(int source, struct mother_hold *mother) {
	char buff[512], *ptr;
	FILE *outfile;

	ptr = getenv("HOME");
	strcpy(buff, ptr);
	strcat(buff, "/.robot_race/world.brd");
	outfile = fopen(buff, "a");
	if(outfile == NULL) {
		return;
	}

	readn(source, buff, 6);
/*	g_print("element placement: module(%d) @ (%ld, %ld), instance %d\n",
		buff[0],
		(long)(buff[1] * 250 + buff[2]),
		(long)(buff[3] * 250 + buff[4]),
		buff[5]);
*/	sprintf(buff, "%d %ld %ld %d\n",
		buff[0],
		(long)(buff[1] * 250 + buff[2]),
		(long)(buff[3] * 250 + buff[4]),
		buff[5]);

	fputs(buff, outfile);
	fclose(outfile);
}

void case_module_list(int source, struct mother_hold *mother) {
	char buff[512];
	int num_of_elements, length, id_number, ctr;
	node_user_ptr new_node;
	node_user_ptr slider;
	node_user_ptr kill_me;

	/* remove current element listing if there is one */
	slider = mother->server_head;
	while(slider) {
		kill_me = slider;
		slider = slider->next_user;
		g_free(kill_me->nick);
		g_free(kill_me);
	}
	mother->server_head = NULL;

	readn(source, buff, 1);
	num_of_elements = buff[0];

//	g_print("Element Listing:\n");
	for(ctr = 0; ctr < num_of_elements; ctr ++) {
		readn(source, buff, 2);
		id_number = buff[0];
		length = buff[1];
		readn(source, buff, length);

//		g_print("element: #%d, '%s'\n", id_number, buff);
	
		/* We have our next element, lets make a node and attach it */
		new_node = (node_user_ptr) calloc (1, sizeof(node_user_type));
		new_node->id = id_number;
		new_node->nick = g_strdup(buff);
		new_node->next_user = NULL;

		/* find end of link list to attach this node */
		slider = mother->server_head;
		if(slider != NULL) {
			while(slider->next_user != NULL) {
				slider = slider->next_user;
			}
		}

		if(slider == NULL) {
			mother->server_head = new_node;
			/* I don't think this is possible */
		}
		else {
			slider->next_user = new_node;
		}
	}
//	g_print("--- end of list ---\n");

/*
	strcpy(buff, "/load ");
	strcat(buff, getenv("HOME"));
	strcat(buff, "/.robot_race/client.com");
*/
}

void case_spawning_robot(int source, struct mother_hold *mother) {
	char buff[9];
	int userptr;
	int new_is_virtual, new_facing_direction;
	long new_x, new_y;

	readn(source, buff, 7);
	userptr = buff[0];
	new_is_virtual = buff[1];
	new_facing_direction = buff[2];
	new_x = (buff[3] * 250) + buff[4];
	new_y = (buff[5] * 250) + buff[6];

	g_print("Robot #%d, virtual %d, facing %d, x %ld, y %ld\n",
		userptr, new_is_virtual, new_facing_direction,
		new_x, new_y);

//	update_user_list(mother);
}

void case_admin_access(int source, struct mother_hold *mother) {
	char answer[1];

	readn(source, answer, 1);

	if(answer[0]) {
		printf("Admin access has been GRANTED.\n");
		gtk_widget_show(mother->server_form.tab1.access_vbox);
	}
	else {
		printf("Admin access has been DENIED.\n");
	}
}

void case_nick(int source, struct mother_hold *mother) {
	char buff[512], newnick[512];
	int userptr, length, ctr;
	node_user_ptr slider;

	readn(source, buff, 1);
	userptr = buff[0];

	/* Read off the length of the new nick */
	readn(source, buff, 1);
	length = buff[0];

	/* Copy off the user's nick */
	for(ctr = 0; ctr < length; ctr ++) {
		readn(source, buff, 1);
		newnick[ctr] = buff[0];
	}

	/* Fix the window title if this is the current users nick */
	if(userptr == mother->whoami) {
		strcpy(buff, RRCLIENT_VER);
		strcat(buff, " - (");
		strcat(buff, newnick);
		strcat(buff, ")");
		gtk_window_set_title(
				GTK_WINDOW (mother->main_form.main_window), buff);
	}

	/* We need to update this users entry */
	slider = mother->user_head;
	while(slider->id != userptr) {
		slider = slider->next_user;
	}
	g_free(slider->nick);
	slider->nick = g_strdup(newnick);

	update_user_list(mother);
}

void case_error(int source, struct mother_hold *mother) {
	char buff[512];

	readn(source, buff, 1);

	printf("Error: ");
	show_as_hex(1, buff);
	printf(" has occurred.\n");
}

void case_quit(int source, struct mother_hold *mother) {
	char buff[512];
	int userptr;
	node_user_ptr slider;
	node_user_ptr tmp;

	readn(source, buff, 1);
	userptr = buff[0];

/*
	sprintf(buff, "\nQUIT: %d", userptr);
    gtk_text_insert(
	        GTK_TEXT(mother->main_form.chat_display),
	        NULL, NULL, NULL, buff, strlen(buff));
*/
	/* We need to remove this users entry */
	slider = mother->user_head;
	if(slider != NULL) {
		if(slider->id == userptr) {
			mother->user_head = mother->user_head->next_user;
			g_free(slider->nick);
			g_free(slider);
		}
		else {
			while(slider->next_user != NULL) {
				if(slider->next_user->id == userptr) {
					tmp = slider->next_user;
					slider->next_user = tmp->next_user;
					g_free(tmp->nick);
					g_free(tmp);
				}
				else {
					slider = slider->next_user;
				}
			}
		}
	}

	update_user_list(mother);
}

void case_whisper(int source, struct mother_hold *mother) {
	unsigned char buff[512], identity[512];
	int userptr, messlen;

	readn(source, buff, 1);
	userptr = buff[0];
	readn(source, buff, 1);
	messlen = (unsigned char)buff[0];
	readn(source, buff, messlen);

	if((messlen > 0) && (messlen < 512)) {
		strcpy(identity, "\n");
		strcat(identity, "<* ");
		strcat(identity, find_user(userptr, mother));
		strcat(identity, " *> ");
		strcat(identity, buff);

//		printf("%s", identity);
//		waddstr(mother->middle, identity);
//		wrefresh(mother->middle);
	    gtk_text_insert(
		        GTK_TEXT(mother->main_form.chat_display),
		        NULL, NULL, NULL, identity, strlen(identity));
	}
}

void case_emote(int source, struct mother_hold *mother) {
	unsigned char buff[512], identity[512];
	int userptr, messlen;

	readn(source, buff, 1);
	userptr = buff[0];
	readn(source, buff, 1);
	messlen = (unsigned char)buff[0];
	readn(source, buff, messlen);

	if((messlen > 0) && (messlen < 512)) {
		strcpy(identity, "\n");
		strcat(identity, "<");
		strcat(identity, find_user(userptr, mother));
		strcat(identity, " ");
		strcat(identity, buff);

//		printf("%s", identity);
//		waddstr(mother->middle, identity);
//		wrefresh(mother->middle);
	    gtk_text_insert(
		        GTK_TEXT(mother->main_form.chat_display),
		        NULL, NULL, NULL, identity, strlen(identity));
	}
}

void case_talk(int source, struct mother_hold *mother) {
	unsigned char buff[512], identity[512];
	int userptr, messlen;

	readn(source, buff, 1);
	userptr = buff[0];
	readn(source, buff, 1);
	messlen = (unsigned char)buff[0];
	readn(source, buff, messlen);

	if((messlen > 0) && (messlen < 512)) {
		strcpy(identity, "\n<");
		strcat(identity, find_user(userptr, mother));
		strcat(identity, "> ");
		strcat(identity, buff);

//		printf("%s", identity);
//		waddstr(mother->middle, identity);
//		wrefresh(mother->middle);
	    gtk_text_insert(
		        GTK_TEXT(mother->main_form.chat_display),
		        NULL, NULL, NULL, identity, strlen(identity));
	}
}

void case_join(int source, struct mother_hold *mother) {
	char *tmp[3];
	char buff[512];
	int userptr, nicklen;
	node_user_ptr new_node;
	node_user_ptr slider;

	readn(source, buff, 1);
	userptr = buff[0];
	readn(source, buff, 1);
	nicklen = buff[0];
	readn(source, buff, nicklen);

	slider = mother->user_head;
	if(slider != NULL) {
		while(slider->next_user != NULL) {
			slider = slider->next_user;
		}
	}

	if((nicklen > 0) && (nicklen < 100)) {
		/* We have our next user, lets make a node and attach it */
		new_node = (node_user_ptr) calloc (1, sizeof(node_user_type));
		new_node->id = userptr;
		new_node->nick = g_strdup(buff);
		new_node->next_user = NULL;

		if(slider == NULL) {
			mother->user_head = new_node;
			/* I don't think this is possible */
		}
		else {
			slider->next_user = new_node;
		}

		/* Display that the user has joined */
//		printf("** %s has joined.\n", new_node->nick);

		tmp[0] = 0;
		tmp[1] = 0;
		tmp[2] = new_node->nick;
		gtk_clist_append(
				GTK_CLIST(mother->main_form.user_list_clist), tmp);
	}

	update_user_list(mother);
}

/* Returns NULL if the user isn't found in the userlist */
char* find_user(int userptr, struct mother_hold *mother) {
	node_user_ptr slider;

	slider = mother->user_head;
	while(slider != NULL) {
		if(slider->id == userptr) {
			return slider->nick;
		}
		else {
			slider = slider->next_user;
		}
	}

	return "";
}

void case_user_listing(int source, struct mother_hold *mother) {
	char buff[512], *usernick;
	int ctr, userptr, nicklen;
	node_user_ptr new_node;
	node_user_ptr slider;

	usernick = mother->usernick;

//	g_print("User Listing Recieved.\n");

	/* The list is always initially empty, this only happens once */
	mother->user_head = slider = NULL;

	/* Now load the user list */
	readn(source, buff, 1);	/* This is how many users are connected */
	ctr = buff[0];
	while(ctr--) {
		readn(source, buff, 1);
		userptr = buff[0];
		readn(source, buff, 1);
		nicklen = buff[0];
		readn(source, buff, nicklen);

		if((nicklen > 0) && (nicklen < 100)) {
			/* We have our next user, lets make a node and attach it */
			new_node = (node_user_ptr) calloc (1, sizeof(node_user_type));
			new_node->id = userptr;
			new_node->nick = g_strdup(buff);
			new_node->next_user = NULL;

			if(mother->user_head == NULL) {
				mother->user_head = new_node;
			}
			else {
				slider->next_user = new_node;
			}
			slider = new_node;

			if(!strcmp(slider->nick, usernick)) {
				mother->whoami = slider->id;

				strcpy(buff, RRCLIENT_VER);
				strcat(buff, " - (");
				strcat(buff, slider->nick);
				strcat(buff, ")");
				gtk_window_set_title(
						GTK_WINDOW (mother->main_form.main_window), buff);
			}
		}
	}

	update_user_list(mother);
}

/* Locate a user node pointer using the user's id number */
node_user_ptr find_node_user_ptr_from_id(userPtr, mother)
int userPtr;
struct mother_hold *mother;
{
	node_user_ptr user_slider;

	/* Find user using the specified user id number */
	user_slider = mother->user_head;
	while(user_slider != NULL) {
		if(user_slider->id == userPtr) {
			break;
		}

		user_slider = user_slider->next_user;
	}

	return user_slider;
}

void update_user_list(struct mother_hold *mother) {
	node_user_ptr user_slider;
	int user_count = 0;
	char *buff[3];

	buff[0] = 0;
	buff[1] = 0;

	/* Add a line just to make sure the clear isn't clearing an empty list */
	buff[2] = 0;
	gtk_clist_append(
			GTK_CLIST(mother->main_form.user_list_clist), buff);
	/* Clear the clist */
	gtk_clist_clear(GTK_CLIST(mother->main_form.user_list_clist));

//	printf("-------------------------\n");

	/* Count the number of users connected */
	user_slider = mother->user_head;
	while(user_slider) {
//		printf("User Listing: %s\n", user_slider->nick);
//		buff[2] = g_strdup(user_slider->nick);
		buff[2] = user_slider->nick;
		gtk_clist_append(
				GTK_CLIST(mother->main_form.user_list_clist), buff);

		++user_count;
		user_slider = user_slider->next_user;
	}
}

struct node_module_type* find_module_by_server_ref_num(mother, ref_num)
struct mother_hold *mother;
int ref_num;
{
	struct node_user_type *server_mod_slider;
	struct node_module_type *client_mod_slider;

	/* find the server server_mod */
	server_mod_slider = mother->server_head;
	while(server_mod_slider) {
		if(server_mod_slider->id == ref_num) {
			break;
		}
		server_mod_slider = server_mod_slider->next_user;
	}

	/* now find client client_mod which works with this server client_mod */
	client_mod_slider = mother->module_head;
	while(client_mod_slider) {
		if(!strcmp(client_mod_slider->name+1, 
				server_mod_slider->nick+1)) {
			break;
		}
		client_mod_slider = client_mod_slider->next_module;
	}

	/* we now have the client module */
	return client_mod_slider;
}
