#include <gtk/gtk.h>
#include "define.h"
#include "../client_basic_modules/common/module.h"
#include "server_form.h"
#include "edit_layout.h"
#include "update.h"

void display_edit_location_clicked(GtkWidget *widget, gpointer user_data) {
	struct mother_hold *mother;
	int search_ptr;

	mother = (struct mother_hold*)user_data;

	for(search_ptr = 0;
			search_ptr < mother->server_form.tab2.number_of_display_toggles;
			search_ptr++) {
		if(gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(
					mother->server_form.tab2.table_of_display_toggles[
							search_ptr])
				)) {
			/* Change the current location to this toggled button then */
			/* untoggle the button before the user sees it. */
			mother->server_form.tab2.current_display_toggle = search_ptr;

			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(
						mother->server_form.tab2.table_of_display_toggles[
						search_ptr])
					, FALSE);

			/* We are done, exit the loop */
			break;
		}
	}

	/* Check for the double click phenomina and ignore it. */
	if(search_ptr == mother->server_form.tab2.number_of_display_toggles) {
		return;
	}

	/* Do we even need to keep track of the current location? */
	/* Probably not. */
//	g_print("The current display toggle is: %d\n",
//			mother->server_form.tab2.current_display_toggle);

	/* If there is an edit location clicked then we'll do editing */
	if(mother->server_form.tab2.current_edit_toggle != -1) {
		/* Let this edit click add the item wanted by the user to the design node list */
		add_to_layout(mother);

		/* Update the design layout gui */
		update_design_layout(mother, mother->server_form.tab2.current_display_toggle);
	}
}

void edit_toggle_clicked(GtkWidget *widget, gpointer user_data) {
	struct mother_hold *mother;
	int search_ptr; int toggle_count;
	int first, second;

	mother = (struct mother_hold*)user_data;

	/* Is there more than one button toggled? */
	first = second = -1;
	toggle_count = 0;
	for(search_ptr = 0;
			search_ptr < mother->server_form.tab2.number_of_edit_toggles;
			search_ptr++) {
		if(gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(
					mother->server_form.tab2.table_of_edit_toggles[search_ptr])
				)) {
			toggle_count++;
			if(first == -1) {
				first = search_ptr;
			}
			else {
				second = search_ptr;
			}
		}
	}

	if(toggle_count == 0) {
		/* Nothing is toggled, set appropriately */
		//g_print("/* Nothing is toggled, set appropriately */\n");
		mother->server_form.tab2.current_edit_toggle = -1;
	}
	else if(toggle_count == 1) {
		/* One button is toggled so that is our current one. */
		//g_print("/* One button is toggled so that is our current one. */\n");
		mother->server_form.tab2.current_edit_toggle = first;
	}
	else {
		/* Two buttons are toggled, we must have decided to switch */
		//g_print(
		//"/* Two buttons are toggled, we must have decided to switch */\n");
		if(first == mother->server_form.tab2.current_edit_toggle) {
			/* Turn off first, second is our current. */
			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(
						mother->server_form.tab2.table_of_edit_toggles[first])
					, FALSE);
			mother->server_form.tab2.current_edit_toggle = second;
		}
		else {
			/* Turn off second, first is our current. */
			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(
						mother->server_form.tab2.table_of_edit_toggles[second])
					, FALSE);
			mother->server_form.tab2.current_edit_toggle = first;
		}
	}
//	g_print("The current edit toggle is: %d\n",
//			mother->server_form.tab2.current_edit_toggle);
}

/**
 * Create edit table using columns of 8 and as many rows as necessary.
 */
void update_server_form_edit(mother)
struct mother_hold *mother;
{
	GtkWidget *togglebutton;
	GtkWidget *table;
	GtkWidget *gtkpixmap[20800];
	int row, col;
	int max_row, max_col;
	int number_of_images;
	int images_per_module;
	struct node_module_type *module_slider;
	int image_ptr, toggle_ptr;

	/* Count how many small pictures will need to be displayed. */
	number_of_images = 0;
	module_slider = mother->module_head;
	while(module_slider) {
		number_of_images += module_slider->info->number_of_small_images;
		module_slider = module_slider->next_module;
	}
//	g_print("Updating server form: There are %d images to display.\n",
//			number_of_images);
	/* Report errors. */
	if(number_of_images > 150) {
		g_print("ERROR: Number of module images is hardcoded at 150.\n");
		g_print("       This can easily be increased, notify the author.\n");
		g_print("       Editing layout action has been CANCELED.\n");
	}

	/* Determine maximum rows & columns based on number of small images in */
	/* loaded modules. */
	max_col = 8;
	max_row = 1 + number_of_images / max_col;

	/* Destroy the table so everything involved will be destroyed too and */
	/* we won't need to keep track of anything and can just redo it. */
	gtk_widget_destroy(mother->server_form.tab2.edit_table);

	/**
	 * Make the new table.
	 */
	table = gtk_table_new (max_row, max_col, TRUE);
	gtk_widget_ref (table);
	gtk_object_set_data_full (GTK_OBJECT (mother->server_form.main_window),
			"table", table, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table);
	gtk_box_pack_start(
			GTK_BOX(mother->server_form.tab2.edit_vbox), table, TRUE, TRUE, 0);
	mother->server_form.tab2.edit_table = table;

	/**
	 * Add toggle buttons to table
	 */
	mother->server_form.tab2.current_edit_toggle = -1;
	mother->server_form.tab2.current_display_toggle = -1;
	module_slider = mother->module_head;
	images_per_module = module_slider->info->number_of_small_images;
	image_ptr = 0;
	toggle_ptr = 0;
	for(row = 0; row < max_row; row ++) {
		for(col = 0; col < max_col; col ++) {
			if(--number_of_images < 0) {
				break;
			}
			/**
			 * Move to next module is this was the last image in this one
			 */
			while(image_ptr == images_per_module) {
				module_slider = module_slider->next_module;

//				if(module_slider) {
					images_per_module =
							module_slider->info->number_of_small_images;
					image_ptr = 0;
//				}
			}

  			togglebutton = gtk_toggle_button_new();
			gtk_widget_ref (togglebutton);
			gtk_object_set_data_full(
					GTK_OBJECT (mother->server_form.main_window),
					"togglebutton", togglebutton,
					(GtkDestroyNotify) gtk_widget_unref);
			gtk_widget_show (togglebutton);
			/* these are (old_col, new_col)x(old_row, new_row) */
			gtk_table_attach (GTK_TABLE (table), togglebutton,
					col, col+1, row, row+1,
					(GtkAttachOptions) (0),
					(GtkAttachOptions) (0), 0, 0);
			gtk_widget_set_usize (togglebutton, 25, -2);
			gtk_signal_connect(GTK_OBJECT(togglebutton), "clicked",
					GTK_SIGNAL_FUNC(edit_toggle_clicked),
					(gpointer)mother);

			/* Add this toggle to our storage of pointers in mother */
			mother->server_form.tab2.table_of_edit_toggles[toggle_ptr]
					= togglebutton;

			/**
			 * Add a gtkpixmap to the toggle button
			 */
			gtkpixmap[toggle_ptr] = gtk_pixmap_new(
					module_slider->info->list_of_small_images[image_ptr],
					module_slider->info->list_of_small_masks[image_ptr]);
			gtk_widget_show(gtkpixmap[toggle_ptr]);
			gtk_container_add(
					GTK_CONTAINER(togglebutton),
					gtkpixmap[toggle_ptr]);

			/* Increment toggle button counter */
			toggle_ptr++;

			/* Look at next picture */
			image_ptr++;
		}
	}
	mother->server_form.tab2.number_of_edit_toggles = toggle_ptr;

//	g_print("call here just for testing\n");
//	update_server_form_display(mother);
}

/**
 * Create display edit table using useing set rows & cols that are in the
 * spintoggles.
 */
void update_server_form_display(mother)
struct mother_hold *mother;
{
	GtkWidget *togglebutton;
	GtkWidget *table;
	int row, col, max_row, max_col, count, goal;
//	struct node_module_type *module_slider;
	GdkRectangle update_rect;

	/* display the progress bar window to show the loading speed... */
	gtk_window_set_title (GTK_WINDOW(
  			mother->server_form.tab2.design_progress_bar_window),
			"Forming design layout table...");
	gtk_widget_show(mother->server_form.tab2.design_progress_bar_window);

	/* Destroy the table so everything involved will be destroyed too and */
	/* we won't need to keep track of anything and can just redo it. */
	gtk_widget_destroy(mother->server_form.tab2.display_edit_table);

	/**
	 * Make the new table.
	 */
	max_row = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(mother->server_form.tab2.spin_world_button_y));
	max_col = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(mother->server_form.tab2.spin_world_button_x));

	/* 'goal' is needed in order to update the progress bar */
	goal = max_row * max_col;

//	g_print("col=%d, row=%d\n", max_col, max_row);

	table = gtk_table_new (max_row, max_col, TRUE);
	gtk_widget_ref (table);
	gtk_object_set_data_full (GTK_OBJECT (mother->server_form.main_window),
			"table", table, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(
					mother->server_form.tab2.display_edit_scrolledwindow),
		   	table);
	mother->server_form.tab2.display_edit_table = table;

	/**
	 * Add a gtkpixmap to the table location.
	 */
	count = 0;
	for(row = 0; row < max_row; row ++) {
		for(col = 0; col < max_col; col ++) {
			/* Create gtkpixmap to add to location */
			mother->server_form.tab2.table_of_gtkpixmaps[count] =
				gtk_pixmap_new(
						mother->server_form.tab2.empty_square_xpm,
						mother->server_form.tab2.empty_square_mask);
			gtk_widget_show(
					mother->server_form.tab2.table_of_gtkpixmaps[count]);

			/* Create toggle button to add gtkpixmap to */
  			togglebutton = gtk_toggle_button_new();
			gtk_widget_ref (togglebutton);
			gtk_object_set_data_full(
					GTK_OBJECT (mother->server_form.main_window),
					"togglebutton", togglebutton,
					(GtkDestroyNotify) gtk_widget_unref);
			gtk_widget_show (togglebutton);
			/* these are (old_col, new_col)x(old_row, new_row) */
			gtk_table_attach (GTK_TABLE (table), togglebutton,
					col, col+1, row, row+1,
					(GtkAttachOptions) (0),
					(GtkAttachOptions) (0), 0, 0);
//			gtk_widget_set_usize (togglebutton, 25, -2);
			gtk_widget_set_usize (togglebutton, 42, 42);
			gtk_signal_connect(GTK_OBJECT(togglebutton), "clicked",
					GTK_SIGNAL_FUNC(display_edit_location_clicked),
					(gpointer)mother);

			/* Add this toggle to our storage of pointers in mother */
			mother->server_form.tab2.table_of_display_toggles[count]
					= togglebutton;

			/* Add the gtkpixmap to the togglebutton */
			gtk_container_add(
					GTK_CONTAINER(togglebutton),
					mother->server_form.tab2.table_of_gtkpixmaps[count++]);

			/* update the spot which was drawn to */
		    update_rect.x = 0;
			update_rect.y = 0;
			update_rect.width = 250;
			update_rect.height = 100;
			gtk_widget_draw(
					mother->server_form.tab2.design_progress_bar,
					&update_rect);
			gtk_progress_bar_update(
					GTK_PROGRESS_BAR(mother->server_form.tab2.design_progress_bar),
					count / (float)goal);

//			g_print("progress: %f\n", (count / (float)goal));
		}
	}
	mother->server_form.tab2.number_of_display_toggles = count;

	/* hide the progress bar window now that the display is complete... */
	gtk_widget_hide(mother->server_form.tab2.design_progress_bar_window);
}
