/**
 *
 * Author: Travis W. Loyd
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "loadxpms.h"

/* Prototypes */
void form_small_images(void);
void form_large_images(void);

/* Constants */
#define NUM_OF_SMALL_IMAGES 24
#define NUM_OF_LARGE_IMAGES 24

/* Global variables related to module description */
struct initial_info_hold initial_info;
GdkPixmap *list_of_small_images[NUM_OF_SMALL_IMAGES];
GdkBitmap *list_of_small_masks[NUM_OF_SMALL_IMAGES];
GdkPixmap *list_of_large_images[NUM_OF_LARGE_IMAGES];
GdkBitmap *list_of_large_masks[NUM_OF_LARGE_IMAGES];
char *list_of_styles[] = { "element", "only_one" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables used by the module */
int module_reference_number_in_server;
struct mother_hold *mother;

/* Initialization function called first right after module is loaded */
void initialize(struct mother_hold *mother_ptr) {
	mother = mother_ptr;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Set the images in the server */
	form_small_images();
	initial_info.list_of_small_images = list_of_small_images;
	initial_info.list_of_small_masks = list_of_small_masks;
	initial_info.number_of_small_images = NUM_OF_SMALL_IMAGES;
	form_large_images();
	initial_info.list_of_large_images = list_of_large_images;
	initial_info.list_of_large_masks = list_of_large_masks;
	initial_info.number_of_large_images = NUM_OF_LARGE_IMAGES;

	/* A style is required */
	initial_info.list_of_styles = list_of_styles;
	initial_info.number_of_styles = 2;

	/* The display depth for these imags */
	initial_info.display_depth = FLAT_ON_FLOOR;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

void form_small_images(void) {
	GdkColormap *colormap;

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);

    list_of_small_images[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[0],
		NULL, normal_tred_00_xpm);
    list_of_small_images[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[1],
		NULL, normal_tred_01_xpm);
    list_of_small_images[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[2],
		NULL, normal_tred_02_xpm);
    list_of_small_images[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[3],
		NULL, normal_tred_03_xpm);
    list_of_small_images[4] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[4],
		NULL, normal_tred_04_xpm);
    list_of_small_images[5] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[5],
		NULL, normal_tred_05_xpm);
    list_of_small_images[6] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[6],
		NULL, normal_tred_06_xpm);
    list_of_small_images[7] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[7],
		NULL, normal_tred_07_xpm);
    list_of_small_images[8] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[8],
		NULL, normal_tred_08_xpm);
    list_of_small_images[9] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[9],
		NULL, normal_tred_09_xpm);
    list_of_small_images[10] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[10],
		NULL, normal_tred_10_xpm);
    list_of_small_images[11] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[11],
		NULL, normal_tred_11_xpm);

    list_of_small_images[12] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[12],
		NULL, normal_tred_12_xpm);
    list_of_small_images[13] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[13],
		NULL, normal_tred_13_xpm);
    list_of_small_images[14] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[14],
		NULL, normal_tred_14_xpm);
    list_of_small_images[15] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[15],
		NULL, normal_tred_15_xpm);

    list_of_small_images[16] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[16],
		NULL, normal_tred_16_xpm);
    list_of_small_images[17] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[17],
		NULL, normal_tred_17_xpm);
    list_of_small_images[18] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[18],
		NULL, normal_tred_18_xpm);
    list_of_small_images[19] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[19],
		NULL, normal_tred_19_xpm);

    list_of_small_images[20] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[20],
		NULL, normal_tred_20_xpm);
    list_of_small_images[21] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[21],
		NULL, normal_tred_21_xpm);
    list_of_small_images[22] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[22],
		NULL, normal_tred_22_xpm);
    list_of_small_images[23] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[23],
		NULL, normal_tred_23_xpm);
}

void form_large_images(void) {
	GdkColormap *colormap;

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);

    list_of_large_images[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[0],
		NULL, big_normal_tred_00_xpm);
    list_of_large_images[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[1],
		NULL, big_normal_tred_01_xpm);
    list_of_large_images[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[2],
		NULL, big_normal_tred_02_xpm);
    list_of_large_images[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[3],
		NULL, big_normal_tred_03_xpm);
    list_of_large_images[4] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[4],
		NULL, big_normal_tred_04_xpm);
    list_of_large_images[5] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[5],
		NULL, big_normal_tred_05_xpm);
    list_of_large_images[6] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[6],
		NULL, big_normal_tred_06_xpm);
    list_of_large_images[7] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[7],
		NULL, big_normal_tred_07_xpm);
    list_of_large_images[8] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[8],
		NULL, big_normal_tred_08_xpm);
    list_of_large_images[9] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[9],
		NULL, big_normal_tred_09_xpm);
    list_of_large_images[10] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[10],
		NULL, big_normal_tred_10_xpm);
    list_of_large_images[11] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[11],
		NULL, big_normal_tred_11_xpm);

    list_of_large_images[12] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[12],
		NULL, big_normal_tred_12_xpm);
    list_of_large_images[13] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[13],
		NULL, big_normal_tred_13_xpm);
    list_of_large_images[14] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[14],
		NULL, big_normal_tred_14_xpm);
    list_of_large_images[15] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[15],
		NULL, big_normal_tred_15_xpm);

    list_of_large_images[16] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[16],
		NULL, big_normal_tred_16_xpm);
    list_of_large_images[17] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[17],
		NULL, big_normal_tred_17_xpm);
    list_of_large_images[18] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[18],
		NULL, big_normal_tred_18_xpm);
    list_of_large_images[19] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[19],
		NULL, big_normal_tred_19_xpm);

    list_of_large_images[20] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[20],
		NULL, big_normal_tred_20_xpm);
    list_of_large_images[21] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[21],
		NULL, big_normal_tred_21_xpm);
    list_of_large_images[22] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[22],
		NULL, big_normal_tred_22_xpm);
    list_of_large_images[23] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[23],
		NULL, big_normal_tred_23_xpm);
}

/**
 * This is the reference number in the server of the module which
 * has the same name as this client module.
 */
void set_reference_number(int module_reference_number_in_server) {
	module_reference_number_in_server = module_reference_number_in_server;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

void init_start_game(server_info)
struct server_info_hold *server_info;
{
	g_print("[client: BASIC_TRED] Initializing module...\n");
}
