/**
 *
 * Author: Travis W. Loyd
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "loadxpms.h"

/* Prototypes */
void form_small_images(void);
void form_large_images(void);

/* Constants */
#define NUM_OF_SMALL_IMAGES 4
#define NUM_OF_LARGE_IMAGES 4

/* Global variables related to module description */
struct initial_info_hold initial_info;
GdkPixmap *list_of_small_images[NUM_OF_SMALL_IMAGES];
GdkBitmap *list_of_small_masks[NUM_OF_SMALL_IMAGES];
GdkPixmap *list_of_large_images[NUM_OF_LARGE_IMAGES];
GdkBitmap *list_of_large_masks[NUM_OF_LARGE_IMAGES];
char *list_of_styles[] = { "element", "multi" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables used by the module */
int module_reference_number_in_server;
struct mother_hold *mother;

/* Initialization function called first right after module is loaded */
void initialize(struct mother_hold *mother_ptr) {
	mother = mother_ptr;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Set the images in the server */
	form_small_images();
	initial_info.list_of_small_images = list_of_small_images;
	initial_info.list_of_small_masks = list_of_small_masks;
	initial_info.number_of_small_images = NUM_OF_SMALL_IMAGES;
	form_large_images();
	initial_info.list_of_large_images = list_of_large_images;
	initial_info.list_of_large_masks = list_of_large_masks;
	initial_info.number_of_large_images = NUM_OF_LARGE_IMAGES;

	/* A style is required */
	initial_info.list_of_styles = list_of_styles;
	initial_info.number_of_styles = 2;

	/* The display depth for these imags */
	initial_info.display_depth = SIT_ON_FLOOR;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

void form_small_images(void) {
	GdkColormap *colormap;

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);

    list_of_small_images[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[0],
		NULL, basic_wall_0_xpm);
    list_of_small_images[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[1],
		NULL, basic_wall_1_xpm);
    list_of_small_images[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[2],
		NULL, basic_wall_2_xpm);
    list_of_small_images[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[3],
		NULL, basic_wall_3_xpm);
}

void form_large_images(void) {
	GdkColormap *colormap;

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);

    list_of_large_images[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[0],
		NULL, big_basic_wall_0_xpm);
    list_of_large_images[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[1],
		NULL, big_basic_wall_1_xpm);
    list_of_large_images[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[2],
		NULL, big_basic_wall_2_xpm);
    list_of_large_images[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_large_masks[3],
		NULL, big_basic_wall_3_xpm);
}

/**
 * This is the reference number in the server of the module which
 * has the same name as this client module.
 */
void set_reference_number(int module_reference_number_in_server) {
	module_reference_number_in_server = module_reference_number_in_server;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

void init_start_game(server_info)
struct server_info_hold *server_info;
{
	g_print("[client: BASIC_WALL] Initializing module...\n");
}
