/**
 *
 * Author: Travis W. Loyd
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "loadxpms.h"
#include "../../client/src/netutil.h"
#include "../../server_basic_modules/common/movement.h"

/* Prototypes */
void form_small_images(void);
void form_large_images(void);
void draw_initial_archive_location(struct mother_hold*);
void draw_archive_location(struct mother_hold*, int);
void initial_display_of_robots(struct mother_hold*);
void move_robot(struct mother_hold*, int);
void animate_robot(struct mother_hold*, struct node_robot_type*,
		int, int, int, int, int, int, int);
int find_module_id_from_name(struct mother_hold*, gchar*);
void shutdown_a_robot(struct mother_hold*, int);
void spawn_a_robot(struct mother_hold*, int);
void hide_robot(struct mother_hold*, int);
void powerup(struct mother_hold*, int);
void set_archive_location(struct mother_hold*, long, long, long);
void set_virtual(struct mother_hold*, long);
void set_nonvirtual(struct mother_hold*, long);

/* Constants */
#define NUM_OF_SMALL_IMAGES 0
#define NUM_OF_LARGE_IMAGES 0

/* Global variables related to module description */
struct initial_info_hold initial_info;
GdkPixmap *list_of_small_images[NUM_OF_SMALL_IMAGES];
GdkBitmap *list_of_small_masks[NUM_OF_SMALL_IMAGES];
GdkPixmap *shutdown_robots_pixmap[4];
GdkBitmap *shutdown_robots_bitmap[4];
GdkPixmap *spawn_location_pixmap;
GdkBitmap *spawn_location_bitmap;
GdkPixmap *virtual_robots_pixmap[4];
GdkBitmap *virtual_robots_bitmap[4];
GdkPixmap *nonvirtual_robots_pixmap[4];
GdkBitmap *nonvirtual_robots_bitmap[4];
char *list_of_styles[] = { "only_one" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables used by the module */
int module_reference_number_in_server;
struct mother_hold *mother;

/* Initialization function called first right after module is loaded */
void initialize(struct mother_hold *mother_ptr) {
	mother = mother_ptr;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Set the images in the server */
	form_small_images();
	initial_info.list_of_small_images = list_of_small_images;
	initial_info.list_of_small_masks = list_of_small_masks;
	initial_info.number_of_small_images = NUM_OF_SMALL_IMAGES;
	form_large_images();
//	initial_info.nonvirtual_robots_pixmap = nonvirtual_robots_pixmap;
//	initial_info.nonvirtual_robots_bitmap = nonvirtual_robots_bitmap;
	initial_info.number_of_large_images = NUM_OF_LARGE_IMAGES;

	/* A style is required */
	initial_info.list_of_styles = list_of_styles;
	initial_info.number_of_styles = 1;

	/* The display depth for these imags */
	initial_info.display_depth = 0;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

void form_small_images(void) {
/*
	GdkColormap *colormap;

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    list_of_small_images[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &list_of_small_masks[0],
		NULL, basic_pit_xpm);
*/
}

void form_large_images(void) {
	GdkColormap *colormap;

	/* nonvirtual robots */

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    nonvirtual_robots_pixmap[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &nonvirtual_robots_bitmap[0],
		NULL, default_nonvirtual_robot_0_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    nonvirtual_robots_pixmap[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &nonvirtual_robots_bitmap[1],
		NULL, default_nonvirtual_robot_1_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    nonvirtual_robots_pixmap[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &nonvirtual_robots_bitmap[2],
		NULL, default_nonvirtual_robot_2_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    nonvirtual_robots_pixmap[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &nonvirtual_robots_bitmap[3],
		NULL, default_nonvirtual_robot_3_xpm);

	/* virtual robots */

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    virtual_robots_pixmap[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &virtual_robots_bitmap[0],
		NULL, default_virtual_robot_0_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    virtual_robots_pixmap[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &virtual_robots_bitmap[1],
		NULL, default_virtual_robot_1_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    virtual_robots_pixmap[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &virtual_robots_bitmap[2],
		NULL, default_virtual_robot_2_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    virtual_robots_pixmap[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &virtual_robots_bitmap[3],
		NULL, default_virtual_robot_3_xpm);

	/* spawn location */

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    spawn_location_pixmap = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &spawn_location_bitmap,
		NULL, spawn_location_xpm);

	/* shutdown location */

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    shutdown_robots_pixmap[0] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &shutdown_robots_bitmap[0],
		NULL, default_shutdown_robot_0_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    shutdown_robots_pixmap[1] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &shutdown_robots_bitmap[1],
		NULL, default_shutdown_robot_1_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    shutdown_robots_pixmap[2] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &shutdown_robots_bitmap[2],
		NULL, default_shutdown_robot_2_xpm);

	colormap = gtk_widget_get_colormap(mother->server_form.main_window);
    shutdown_robots_pixmap[3] = gdk_pixmap_colormap_create_from_xpm_d(
		NULL, colormap, &shutdown_robots_bitmap[3],
		NULL, default_shutdown_robot_3_xpm);
}

/**
 * This is the reference number in the server of the module which
 * has the same name as this client module.
 */
void set_reference_number(int module_reference_number_in_server) {
	module_reference_number_in_server = module_reference_number_in_server;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

void init_start_game(server_info)
struct server_info_hold *server_info;
{
	g_print("[client: BASIC_ROBOT] Initializing module...\n");
}

void network_command_handler(mother, who_did_it)
struct mother_hold *mother;
gchar *who_did_it;
{
	char buff[512], tmp[512];
	int instance, command;

//	g_print("[client: BASIC_ROBOT] Handling network command called by %d.\n",
//			who_did_it[0]);

	readn(mother->socket, buff, 2);
	g_print("[client: BASIC_ROBOT] (%d) This instance is %d, and command is %d.\n",
			who_did_it[0], buff[0], buff[1]);

	instance = buff[0];
	command = buff[1];

	if(who_did_it[0] == find_module_id_from_name(mother, "s_basic_pit")) {
//		g_print("Robot %d has been destroyed and the result is ",
//				buff[0]);
		sprintf(tmp, "Robot %d has been destroyed and the result is ",
				buff[0]);
		gtk_widget_show(mother->main_form.news_form);
        gtk_text_insert(
                GTK_TEXT(mother->main_form.text_listing),
                NULL, NULL, NULL, tmp,
                strlen(tmp));

		/* assume the destroying was true */
		hide_robot(mother, buff[0]);

		/* but still need to read off the value */
		readn(mother->socket, buff, 1);
		g_print("%d\n", buff[0]);

	}
	else if(who_did_it[0] == find_module_id_from_name(mother, "s_basic_checkpoint")) {
		switch(command) {
			case 0:	/* a checkpoint has been tagged */
				readn(mother->socket, buff, 1);
//				g_print("Robot %d has touched checkpoint #%d\n", instance, buff[0]);
				sprintf(tmp, "Robot %d has touched checkpoint #%d\n", instance, buff[0]);
				gtk_widget_show(mother->main_form.news_form);
	            gtk_text_insert(
	                    GTK_TEXT(mother->main_form.text_listing),
	                    NULL, NULL, NULL, tmp,
	                    strlen(tmp));
				break;
			case 1:
//				g_print("Robot %d has won the game! Congratulations!!!\n", instance);
				sprintf(tmp, "Robot %d has won the game! Congratulations!!!\n", instance);
				gtk_widget_show(mother->main_form.news_form);
	            gtk_text_insert(
	                    GTK_TEXT(mother->main_form.text_listing),
	                    NULL, NULL, NULL, tmp,
	                    strlen(tmp));
				break;
		}
	}
	else if(who_did_it[0] == find_module_id_from_name(mother, "s_basic_repair")) {
		switch(command) {
			case 0:	/* spawn a robot */
				readn(mother->socket, buff, 2);
				set_archive_location(mother, instance, buff[0], buff[1]);
				break;
		}
	}
	else if(who_did_it[0] == find_module_id_from_name(mother, "s_robot")) {
		switch(command) {
			case 0:	/* spawn a robot */
				spawn_a_robot(mother, buff[0]);
				break;
			case 1:
				g_print("[client: BASIC_ROBOT] display initial archive location\n");
				draw_initial_archive_location(mother);
//				draw_archive_location(mother, instance);
				break;
			case 2:
				g_print("[client: BASIC_ROBOT] display all the robots here\n");
				initial_display_of_robots(mother);
				break;
			case 3:
				if(instance == mother->whoami) {
					draw_archive_location(mother, instance);
					gtk_widget_show(mother->main_form.spawn_select_form);
				}
				else {
					g_print("WAITING: on player %d to send in their spawn direction.\n",
							buff[0]);
				}
				break;
			case 4:
//				g_print("[client: BASIC_ROBOT] SHUTTING DOWN %d\n", instance);
				sprintf(tmp, "[client: BASIC_ROBOT] SHUTTING DOWN %d\n", instance);
				gtk_widget_show(mother->main_form.news_form);
	            gtk_text_insert(
	                    GTK_TEXT(mother->main_form.text_listing),
	                    NULL, NULL, NULL, tmp,
	                    strlen(tmp));
				shutdown_a_robot(mother, instance);
				break;
			case 5:
//				g_print("[client: BASIC_ROBOT] RESTARTING ROBOT %d\n", instance);
				sprintf(tmp, "[client: BASIC_ROBOT] RESTARTING ROBOT %d\n", instance);
				gtk_widget_show(mother->main_form.news_form);
	            gtk_text_insert(
	                    GTK_TEXT(mother->main_form.text_listing),
	                    NULL, NULL, NULL, tmp,
	                    strlen(tmp));
				powerup(mother, instance);
				break;
			case 6:
				/* robot is now virtual */
				set_virtual(mother, instance);
				g_print("Setting robot as virtual %d\n", instance);
				break;
			case 7:
				/* robot is now non-virtual */
				set_nonvirtual(mother, instance);
				g_print("Setting robot as nonvirtual %d\n", instance);
				break;
		}
	}
	else {
		/* this handles commands from either 's_p_card' or 's_basic_gear' */
		switch(buff[1]) {
			case 0:
				move_robot(mother, buff[0]);
				break;
		}
	}
}

void set_virtual(mother, roboNum)
struct mother_hold *mother;
long roboNum;
{
	struct node_robot_type* robot_slider;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			robot_slider->is_virtual = TRUE;
			break;
		}

		robot_slider = robot_slider->next_robot;
	}
}

void set_nonvirtual(mother, roboNum)
struct mother_hold *mother;
long roboNum;
{
	struct node_robot_type* robot_slider;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			robot_slider->is_virtual = FALSE;
			break;
		}

		robot_slider = robot_slider->next_robot;
	}
}

void spawn_a_robot(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	struct node_robot_type *robot_slider;
	char buff[512];
	GdkGC *gc;
	GdkRectangle update_rect;

	readn(mother->socket, buff, 2);

g_print("spawning a robot\n");
	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}
		robot_slider = robot_slider->next_robot;
	}

	/* move robot to its archive location */
	robot_slider->current_x = robot_slider->archive_x;
	robot_slider->current_y = robot_slider->archive_y;
	robot_slider->is_virtual = buff[0];
	robot_slider->facing = buff[1];

	/* clear spot robot will spawn on to remove archive location indicator */
	hide_robot(mother, robot_slider->id);

	g_print("Drawing robot %d, at %ld, %ld\n",
			robot_slider->id,
			robot_slider->current_x,
			robot_slider->current_y);

	/* display robot in its new location */
	if(buff[0]) {
		g_print("Spawning robot as virtual\n");

		gc = gdk_gc_new(virtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				virtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				virtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	else {
		g_print("Spawning robot as non-virtual\n");

		gc = gdk_gc_new(nonvirtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				nonvirtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				nonvirtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);
	
g_print("drew the new location\n");
	/* update the new location */
    update_rect.x = (robot_slider->current_x * 40);
    update_rect.y = (robot_slider->current_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void move_robot(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	struct node_robot_type *robot_slider;
	char buff[512];
	int old_facing, new_facing;
	int old_x, new_x;
	int old_y, new_y;

	readn(mother->socket, buff, 1);

	if(buff[0] >= 10) {
		g_print("[client: BASIC_ROBOT] movement failed: %d\n",
				buff[0] - 10);
		return;
	}
	g_print("[client: BASIC_ROBOT] robot %d, movement %d\n",
			roboNum, buff[0]);

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}
		robot_slider = robot_slider->next_robot;
	}

	new_x = old_x = robot_slider->current_x;
	new_y = old_y = robot_slider->current_y;
	new_facing = old_facing = robot_slider->facing;

	switch(buff[0]) {
		case U_TURN:
			new_facing = (new_facing + 1) % 4;
			new_facing = (new_facing + 1) % 4;
			break;
		case ROTATE_LEFT:
			new_facing = (new_facing + 1) % 4;
			new_facing = (new_facing + 1) % 4;
			new_facing = (new_facing + 1) % 4;
			break;
		case ROTATE_RIGHT:
			new_facing = (new_facing + 1) % 4;
			break;
		case BACK_UP:
			switch(old_facing) {
				case 0: ++new_y; break;
				case 1: --new_x; break;
				case 2: --new_y; break;
				case 3: ++new_x; break;
			}
			break;
		case FORWARD:
			switch(old_facing) {
				case 0: --new_y; break;
				case 1: ++new_x; break;
				case 2: ++new_y; break;
				case 3: --new_x; break;
			}
			break;
		case NORTH:
			--new_y;
			break;
		case EAST:
			++new_x;
			break;
		case SOUTH:
			++new_y;
			break;
		case WEST:
			--new_x;
			break;
	}

	robot_slider->current_x = new_x;
	robot_slider->current_y = new_y;
	robot_slider->facing = new_facing;

	animate_robot(mother, robot_slider, buff[0],
			old_x, old_y, old_facing, new_x, new_y, new_facing);
}

void animate_robot(mother, robot_slider, move_type,
		old_x, old_y, old_facing, new_x, new_y, new_facing)
struct mother_hold *mother;
struct node_robot_type *robot_slider;
int move_type;
int old_x;
int old_y;
int old_facing;
int new_x;
int new_y;
int new_facing;
{
	GdkRectangle update_rect;
	GdkGC *gc;

	/* animate this robot movement, for now though... just erase current */
	/* location and draw in new location */

	/* erase current location */

	gc = gdk_gc_new(
			mother->main_form.game_display_pixmap_backup);
    gdk_gc_set_clip_mask(
			gc,
			mother->server_form.tab2.empty_square_mask);
	gdk_gc_set_clip_origin(gc,
			40 * old_x, /* destination */
			40 * old_y); /* destination */
	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			mother->main_form.game_display_pixmap_backup,
			40 * old_x, /* from */
			40 * old_y, /* from */
			40 * old_x, /* destination */
			40 * old_y, /* destination */
			40, 40 /* size of image */);

	/* draw the new location */

	g_print("Drawing robot %d, at %ld, %ld\n",
			robot_slider->id,
			robot_slider->current_x,
			robot_slider->current_y);

	if(robot_slider->is_shutdown) {
		gc = gdk_gc_new(shutdown_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				shutdown_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				shutdown_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	else if(robot_slider->is_virtual) {
		gc = gdk_gc_new(virtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				virtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				virtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	else {
		gc = gdk_gc_new(nonvirtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				nonvirtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				nonvirtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);

	/* update the spot which was cleared */
    update_rect.x = (old_x * 40);
    update_rect.y = (old_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
	
	/* update the new location */
    update_rect.x = (new_x * 40);
    update_rect.y = (new_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void initial_display_of_robots(mother)
struct mother_hold *mother;
{
	GdkRectangle update_rect;
	struct node_robot_type *robot_slider;
	GdkGC *gc;
	int is_virtual;

	g_print("Drawing all robots during initial spawn.\n");

	/**
	 * restore the square which had the initial archive
	 * location marker on it
	 */
	gc = gdk_gc_new(
			mother->main_form.game_display_pixmap_backup);
    gdk_gc_set_clip_mask(
			gc,
			mother->server_form.tab2.empty_square_mask);
	gdk_gc_set_clip_origin(gc,
			40 * mother->initial_archive_location_x, /* destination */
			40 * mother->initial_archive_location_y); /* destination */
	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			mother->main_form.game_display_pixmap_backup,
			40 * mother->initial_archive_location_x, /* from */
			40 * mother->initial_archive_location_y, /* from */
			40 * mother->initial_archive_location_x, /* destination */
			40 * mother->initial_archive_location_y, /* destination */
			40, 40 /* size of image */);

	/**
	 * add each robot one by one
	 */
	if(mother->robot_head) {
		if(mother->robot_head->next_robot) {
			is_virtual = TRUE;
		}
		else {
			is_virtual = FALSE;
		}
	}
	robot_slider = mother->robot_head;
	while(robot_slider) {
		g_print("Drawing robot %d, at %ld, %ld\n",
				robot_slider->id,
				robot_slider->current_x,
				robot_slider->current_y);

		robot_slider->is_virtual = is_virtual;

		if(is_virtual) {
			g_print("Spawning robot as virtual\n");
	
			gc = gdk_gc_new(virtual_robots_pixmap[robot_slider->facing]);
			gdk_gc_set_clip_mask(
					gc,
					virtual_robots_bitmap[robot_slider->facing]);
			gdk_gc_set_clip_origin(gc,
					40 * robot_slider->current_x,
					40 * robot_slider->current_y);
			gdk_draw_pixmap(
					mother->main_form.game_display_pixmap, gc,
					virtual_robots_pixmap[robot_slider->facing],
					0, 0,
					40 * robot_slider->current_x,
					40 * robot_slider->current_y,
					40, 40);
		}
		else {
			g_print("Spawning robot as non-virtual\n");
	
			gc = gdk_gc_new(nonvirtual_robots_pixmap[robot_slider->facing]);
			gdk_gc_set_clip_mask(
					gc,
					nonvirtual_robots_bitmap[robot_slider->facing]);
			gdk_gc_set_clip_origin(gc,
					40 * robot_slider->current_x,
					40 * robot_slider->current_y);
			gdk_draw_pixmap(
					mother->main_form.game_display_pixmap, gc,
					nonvirtual_robots_pixmap[robot_slider->facing],
					0, 0,
					40 * robot_slider->current_x,
					40 * robot_slider->current_y,
					40, 40);
		}

		robot_slider = robot_slider->next_robot;
	}

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);

	/* update this location */
    update_rect.x = (mother->initial_archive_location_x * 40);
    update_rect.y = (mother->initial_archive_location_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void draw_initial_archive_location(mother)
struct mother_hold *mother;
{
	GdkGC *gc;

	gc = gdk_gc_new(spawn_location_pixmap);
    gdk_gc_set_clip_mask(
			gc,
			spawn_location_bitmap);
	gdk_gc_set_clip_origin(gc,
			40 * mother->initial_archive_location_x, /* destination */
			40 * mother->initial_archive_location_y); /* destination */

	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			spawn_location_pixmap,
			0, 0, /* location on the FROM pixmap to start copying from */
			40 * mother->initial_archive_location_x, /* destination */
			40 * mother->initial_archive_location_y, /* destination */
			40, 40 /* size of image */);

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);
}

int find_module_id_from_name(mother, module_name)
struct mother_hold *mother;
gchar *module_name;
{
	struct node_user_type *server_slider;

	server_slider = mother->server_head;
	while(server_slider) {
		if(!strcmp(server_slider->nick, module_name)) {
			break;
		}

		server_slider = server_slider->next_user;
	}
	if(!server_slider) {
		g_print("[client: ROBOT] module needed not found '%s', crashing.\n",
				module_name);
		return -1;
	}

	return server_slider->id;
}

void hide_robot(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	struct node_robot_type *robot_slider;
	GdkGC *gc;
	GdkRectangle update_rect;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}
		robot_slider = robot_slider->next_robot;
	}

	/* clear where the robot was */
	gc = gdk_gc_new(
			mother->main_form.game_display_pixmap_backup);
    gdk_gc_set_clip_mask(
			gc,
			mother->server_form.tab2.empty_square_mask);
	gdk_gc_set_clip_origin(gc,
			40 * robot_slider->current_x, /* destination */
			40 * robot_slider->current_y); /* destination */
	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			mother->main_form.game_display_pixmap_backup,
			40 * robot_slider->current_x, /* from */
			40 * robot_slider->current_y, /* from */
			40 * robot_slider->current_x, /* destination */
			40 * robot_slider->current_y, /* destination */
			40, 40 /* size of image */);

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);

	/* update the spot which was cleared */
    update_rect.x = (robot_slider->current_x * 40);
    update_rect.y = (robot_slider->current_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void draw_archive_location(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	GdkRectangle update_rect;
	struct node_robot_type *robot_slider;
	GdkGC *gc;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}

		robot_slider = robot_slider->next_robot;
	}

	gc = gdk_gc_new(spawn_location_pixmap);
    gdk_gc_set_clip_mask(
			gc,
			spawn_location_bitmap);
	gdk_gc_set_clip_origin(gc,
			40 * robot_slider->archive_x, /* destination */
			40 * robot_slider->archive_y); /* destination */

	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			spawn_location_pixmap,
			0, 0, /* location on the FROM pixmap to start copying from */
			40 * robot_slider->archive_x, /* destination */
			40 * robot_slider->archive_y, /* destination */
			40, 40 /* size of image */);

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);

	/* update the spot which was drawn to */
    update_rect.x = (robot_slider->archive_x * 40);
    update_rect.y = (robot_slider->archive_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void set_archive_location(mother, roboNum, x, y)
struct mother_hold *mother;
long roboNum;
long x;
long y;
{
	struct node_robot_type *robot_slider;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}

		robot_slider = robot_slider->next_robot;
	}

	robot_slider->archive_x = x;
	robot_slider->archive_y = y;

	g_print("Updating archive location to %ld, %ld.\n", x, y);
}

void shutdown_a_robot(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	struct node_robot_type *robot_slider;
	GdkGC *gc;
	GdkRectangle update_rect;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}
		robot_slider = robot_slider->next_robot;
	}

	robot_slider->is_shutdown = TRUE;

	/* clear spot robot is located */
	hide_robot(mother, robot_slider->id);
/*
	g_print("Drawing robot %d, at %ld, %ld\n",
			robot_slider->id,
			robot_slider->current_x,
			robot_slider->current_y);
*/
	/* display robot as shutdown */
	gc = gdk_gc_new(shutdown_robots_pixmap[robot_slider->facing]);
	gdk_gc_set_clip_mask(
			gc,
			shutdown_robots_bitmap[robot_slider->facing]);
	gdk_gc_set_clip_origin(gc,
			40 * robot_slider->current_x,
			40 * robot_slider->current_y);
	gdk_draw_pixmap(
			mother->main_form.game_display_pixmap, gc,
			shutdown_robots_pixmap[robot_slider->facing],
			0, 0,
			40 * robot_slider->current_x,
			40 * robot_slider->current_y,
			40, 40);

	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);
	
	/* update the new location */
    update_rect.x = (robot_slider->current_x * 40);
    update_rect.y = (robot_slider->current_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}

void powerup(mother, roboNum)
struct mother_hold *mother;
int roboNum;
{
	struct node_robot_type *robot_slider;
	GdkGC *gc;
	GdkRectangle update_rect;

	robot_slider = mother->robot_head;
	while(robot_slider) {
		if(robot_slider->id == roboNum) {
			break;
		}
		robot_slider = robot_slider->next_robot;
	}

	robot_slider->is_shutdown = FALSE;

	/* clear spot robot will spawn on to remove archive location indicator */
	hide_robot(mother, robot_slider->id);

	/* display robot in its new location */
	if(robot_slider->is_virtual) {
		gc = gdk_gc_new(virtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				virtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				virtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	else {
		gc = gdk_gc_new(nonvirtual_robots_pixmap[robot_slider->facing]);
		gdk_gc_set_clip_mask(
				gc,
				nonvirtual_robots_bitmap[robot_slider->facing]);
		gdk_gc_set_clip_origin(gc,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y);
		gdk_draw_pixmap(
				mother->main_form.game_display_pixmap, gc,
				nonvirtual_robots_pixmap[robot_slider->facing],
				0, 0,
				40 * robot_slider->current_x,
				40 * robot_slider->current_y,
				40, 40);
	}
	gtk_pixmap_set(GTK_PIXMAP(
			mother->main_form.game_display_gtkpixmap),
			mother->main_form.game_display_pixmap,
			mother->main_form.game_display_bitmap);
	
	/* update the new location */
    update_rect.x = (robot_slider->current_x * 40);
    update_rect.y = (robot_slider->current_y * 40);
    update_rect.width = 40;
    update_rect.height = 40;
    gtk_widget_draw(mother->main_form.game_display_gtkpixmap,
			&update_rect);
}
