#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>
#include "../common/server.h"
#include "../common/module.h"
#include "display.h"

void display_help(void);
int load_module(struct server_info_hold*, char*);
int unload_module(struct server_info_hold*, char*);

/**
 * Displays the information all modules provided when they were loaded.
 */
int display_module_info_all_short(server_info)
struct server_info_hold *server_info;
{
	node_module_ptr module_slider;

	printf("## Listing Modules\n");

	/* Find the module node in the linked list */
	module_slider = server_info->module_head;
	while(module_slider) {
		/* Display the module information in short form */
		display_module_info_short(server_info, module_slider);

		module_slider = module_slider->next_module;
	}

	printf("## End Module Listing\n");

	return 0;
}

/**
 * Displays the information all modules provided when they were loaded.
 */
int display_module_info_all_long(server_info)
struct server_info_hold *server_info;
{
	node_module_ptr module_slider;

	printf("## Listing Modules\n");

	/* Find the module node in the linked list */
	module_slider = server_info->module_head;
	while(module_slider) {
		/* Display the module information in short form */
		display_module_info_long(server_info, module_slider);

		module_slider = module_slider->next_module;
	}

	printf("## End Module Listing\n");

	return 0;
}

/**
 * Displays the information the module provided when it was loaded.
 * - Long Form -
 */
int display_module_info_long(server_info, module_name)
struct server_info_hold *server_info;
node_module_ptr module_name;
{
	int ctr, loop;

	printf("$- [%s v%s]\n"
			, module_name->name, module_name->info->version);
	printf("$- Author: %s\n", module_name->info->author);

	printf("$- Network Commands:"); 
	for(ctr = 0;
			ctr < module_name->info->num_of_network_commands;
			ctr ++) {
		printf(" %d", module_name->info->list_of_network_commands[ctr]);
	}
	printf("\n");

	printf("$- Short Desc:\n%s\n",
			module_name->info->one_line_description);
	printf("$- Multi-Line Description:\n");
	printf("$- -------------------------------------------------------\n");

	for(ctr = 0;
			ctr < module_name->info->num_of_lines_in_multi_description;
			ctr ++) {
		printf("%s\n", module_name->info->multi_description[ctr]);
	}
	printf("$- -------------------------------------------------------\n");

	printf("$- Timing Sessions:");
	for(ctr = 0;
			ctr < module_name->info->num_of_timing_sessions;
			ctr ++) {
		printf(" [%d:",
				module_name->info->list_timing_sessions[ctr]
				.timing_session);

		for(loop = 0;
				loop < module_name->info->list_timing_sessions[ctr].num_of_priorities;
				loop ++) {
			printf(" %d", module_name->info->list_timing_sessions[ctr].
					list_of_priorities[loop]);
		}
		printf("]");
	}
	printf("\n");

	printf("$- Styles:");
	for(ctr = 0;
			ctr < module_name->info->num_of_styles;
			ctr ++) {
		printf(" %s", module_name->info->list_of_styles[ctr]);
	}
	printf("\n");

	return 0;
}

/**
 * Displays the information the module provided when it was loaded.
 * - Short Form -
 */
int display_module_info_short(server_info, module_name)
struct server_info_hold *server_info;
node_module_ptr module_name;
{
	printf("$- [%s v%s]\t"
			, module_name->name, module_name->info->version);
	printf("Author: %s\n", module_name->info->author);

	printf("$- desc: %s\n",
			module_name->info->one_line_description);

	return 0;
}

/* Finds a module according to is char* name and displays the module's info */
/* in long form */
int display_module_info_long_by_name(server_info, module_name)
struct server_info_hold *server_info;
char *module_name;
{
	node_module_ptr module_slider;

	/* Find the module node in the linked list */
	module_slider = server_info->module_head;
	while(strcmp(module_slider->name, module_name)) {
		module_slider = module_slider->next_module;
		if(module_slider == NULL) {
			printf(">! Requested info on a module not loaded\n");
			return 1;
		}
	}

	display_module_info_long(server_info, module_slider);

	return 0;
}

/* Finds a module according to is char* name and displays the module's info */
/* in sort form */
int display_module_info_short_by_name(server_info, module_name)
struct server_info_hold *server_info;
char *module_name;
{
	node_module_ptr module_slider;

	/* Find the module node in the linked list */
	module_slider = server_info->module_head;
	while(strcmp(module_slider->name, module_name)) {
		module_slider = module_slider->next_module;
		if(module_slider == NULL) {
			printf(">! Requested info on a module not loaded\n");
			return 1;
		}
	}

	display_module_info_short(server_info, module_slider);

	return 0;
}
