#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include "../common/server.h"
#include "../common/module.h"
#include "sortlist.h"

/* Execute module functions if there are any */
int call_module_functions_in_order(server_info, sort_list_head)
struct server_info_hold *server_info;
node_sort_ptr sort_list_head;
{
	int (*action_handle_timing)(struct server_info_hold*
			, unsigned int, unsigned int);
	node_sort_ptr kill_slider;
	char buff[512];

	/* Slide through linked list calling module functions as we go */
	/* free up the memory used by the linked list nodes as well */
	kill_slider = sort_list_head;
	while(kill_slider) {
		sort_list_head = kill_slider->next_sort;

		/* Build modules function */
		strcpy(buff, "action_handle_timing");
//		strcpy(buff, "action_");
//		strcat(buff, kill_slider->module->name);
//		strcat(buff, "_handle_timing");

		/* Call action function */
		action_handle_timing = dlsym(kill_slider->module->handle, buff);
		action_handle_timing(server_info
				, server_info->current_timing_session
				, kill_slider->priority_value);
/*
printf("[DEBUG] (execute) Just called: 'timing' in: '%s' @ %d,%d\n",
		kill_slider->module->name
		, server_info->current_timing_session
		, kill_slider->priority_value);
*/
		/* Move to next node */
		kill_slider = sort_list_head;
	}

	return 0;
}

/* Launch all modules which have requested to be active during */
/* this particular timing session. */
int	execute_launch_timing_session(server_info)
struct server_info_hold *server_info;
{
	node_sort_ptr sort_list_head = NULL;
	node_module_ptr module_slider;
	int ctr, loop;

	printf(">> Launching functions in timing session #%d\n",
			server_info->current_timing_session);

	module_slider = server_info->module_head;
	while(module_slider) {
		for(ctr = 0;
			ctr < module_slider->info->num_of_timing_sessions;
			ctr ++) {
			/* Only deal with this module if it is active during this */ 
			/* timing session */
			if(module_slider->info->list_timing_sessions[ctr].timing_session
					== server_info->current_timing_session) {

				for(loop = 0;
					loop < module_slider->info
							->list_timing_sessions[ctr].num_of_priorities;
					loop ++) {
					/* Loop through each priority of this timing session */
					/* For each priority add a node to the sorted list */
					link_sort_next_node(&sort_list_head, module_slider
							, module_slider->info->list_timing_sessions[ctr].
									list_of_priorities[loop]
							, server_info->current_timing_session);
				}
			}
		}

		module_slider = module_slider->next_module;
	}

	/* Execute module functions if there are any */
	call_module_functions_in_order(server_info, sort_list_head);

	/* Free up sort linked list */
	link_clear_list(sort_list_head);

	return TRUE;
}

/* The timing session has already taken place. Check to see if */
/* the modules are all ready to move to the next timing session. */
int execute_is_ready_for_next_timing_session(server_info)
struct server_info_hold *server_info;
{
	int (*is_ready)(struct server_info_hold*);
	node_module_ptr module_slider;
	int all_modules_are_ready;
	char buff[512];

	printf("]] Testing ready modules:\n\t");

	/* By default we expect all modules to be ready */
	all_modules_are_ready = TRUE;

	module_slider = server_info->module_head;
	while(module_slider) {
		/* Create string of the function to call */
		strcpy(buff, "is_ready");
//		strcpy(buff, "is_ready_");
//		strcat(buff, module_slider->name);

		printf(" '%s'", module_slider->name);

		/* See if module is ready to continue */
		is_ready = dlsym(module_slider->handle, buff);
		if(!is_ready(server_info)) {
			printf("(No)");
			all_modules_are_ready = FALSE;
			break;
		}

		printf("(Yes)");

		/* Try out next module if there are anymore */
		module_slider = module_slider->next_module;
	}
	printf("\n");

	/* Quit here if all the modules are not ready */
	if(!all_modules_are_ready) {
		return FALSE;
	}

	/* All modules are ready. */

	/* The game may have been shutdown by setting */
	/* the number_of_timing_sessions variable to */
	/* zero, check for that before advancing the */
	/* timing session */
	if(server_info->number_of_timing_sessions) {
		server_info->current_timing_session =
				(server_info->current_timing_session + 1)
				% server_info->number_of_timing_sessions;
	}
	else {
		return FALSE;
	}

	return TRUE;
}
