#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/define.h"
#include "../common/server.h"
#include "../common/user.h"
#include "../common/modutil.h"

#ifdef WIN32
#include <windows.h>
#include <winsock.h>
#else
#define close_socket close
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>			/* write, read, close */
/*
#include <netinet/in.h>				haven't needed this yet?
*/
#endif /* WIN32 */

/* The other 'say' functions basically find the users to write to, this */
/* function actually does the writing to the socket */
int network_write_to_socket(socket, buffer, num_of_chars)
int socket;
char *buffer;
int num_of_chars;
{
	int nleft, nwritten;

	nleft = num_of_chars;
	while(nleft > 0) {   
		nwritten = send(socket, buffer, nleft, 0);

		if(nwritten < 0) {
			printf("! error: occured during write to socket\n");
		}

		nleft -= nwritten;
		buffer += nwritten;
	}

	return 0;
}

/**
 * Sends a string to a specific user
 */
int network_say_to_user(server_info, buffer, num_of_chars, id_number)
struct server_info_hold *server_info;
char *buffer;
int num_of_chars;
int id_number;
{
	node_user_ptr user_slider;

	/* Find the user node in the linked list */
	user_slider = server_info->user_head;
	while(user_slider->id_number != id_number) {
		if(user_slider == NULL) {
			printf("! tried to write to socket but user node does not exist\n");
			return 1;
		}
		user_slider = user_slider->next_user;
	}

	/* The other two functions do a nick check here to make sure the user */
	/* has identified but for the time being it is assumed that if this */
	/* function is called specifying a user then they want the user to get */
	/* the message so we are not going to stop them. */

	network_write_to_socket(user_slider->socket, buffer, num_of_chars);

	return 0;
}

/**
 * Sends a string to all users who have sent a nick.
 */
int network_say_to_all(server_info, buffer, num_of_chars, id_number)
struct server_info_hold *server_info;
char *buffer;
int num_of_chars;
int id_number;
{
	node_user_ptr user_slider;

	/* Find the user node in the linked list */
	user_slider = server_info->user_head;
	while(user_slider) {
		/* Is it ok to send to this user? */
		if(!user_slider->nick) {
			/* If they haven't sent a nick yet we don't talk to them */
			return 1;
		}
		else {
			/* Send data to this user */
			network_write_to_socket(user_slider->socket, buffer, num_of_chars);
		}

		user_slider = user_slider->next_user;
	}

	return 0;
}

/**
 * Sends a string to all users who have sent a nick except one.
 */
int network_say_to_all_but(server_info, buffer, num_of_chars, id_number)
struct server_info_hold *server_info;
char *buffer;
int num_of_chars;
int id_number;
{
	node_user_ptr user_slider;

	/* Find the user node in the linked list */
	user_slider = server_info->user_head;
	while(user_slider) {
		/* Is it ok to send to this user? */
		if(!user_slider->nick) {
			/* If they haven't sent a nick yet we don't talk to them */
			return 1;
		}
		else {
			/* Avoid sending to the single user here */
			if(user_slider->id_number != id_number) {
				/* Send data to this user */
				network_write_to_socket(
						user_slider->socket, buffer, num_of_chars);
			}
		}

		user_slider = user_slider->next_user;
	}

	return 0;
}
