#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>
#include "../common/server.h"
#include "../common/module.h"
#include "display.h"
#include "listuser.h"
#include "load_unload.h"

void display_help(void);
int load_module(struct server_info_hold*, char*);
int unload_module(struct server_info_hold*, char*);

int parse_stdin(server_info, standard_input)
struct server_info_hold *server_info;
char *standard_input;
{
	char *strptr, *tmp;
//	printf("> parseing: '%s'\n", standard_input);

	strptr = NULL;

	if(!strcmp(standard_input, "load all")) {
		printf(">! loading all available modules\n");
	}
	else if(!strcmp(standard_input, "quit")) {
		printf("request to shutdown recieved via stdin\n");
		exit(0);
	}
	else if(!strcmp(standard_input, "help")) {
		display_help();
	}
	else {
		/* Now see if there are parameters */
		strptr = strstr(standard_input, " ");
		if(strptr) {
			strptr[0] = 0; strptr ++;

			/* If this isn't the end of the string */
			if(strptr) {
				if(!strcmp(standard_input, "load")) {
					if(load_module(server_info, strptr)) {
					}
					else {
						display_module_info_long_by_name(server_info, strptr);
					}
				}
				else if(!strcmp(standard_input, "sload")) {
					if(load_module(server_info, strptr)) {
					}
					else {
					}
				}
				else if(!strcmp(standard_input, "unload")) {
					if(unload_module(server_info, strptr)) {
						printf(">! module failed to unload.\n");
					}
					else {
						printf(">! module unloaded.\n");
					}
				}
				else if(!strcmp(standard_input, "slist")) {
					if(!strcmp(strptr, "all")) {
						display_module_info_all_short(server_info);
					}
					else if(strptr) {
						display_module_info_short_by_name(
								server_info, strptr);
					}
					else {
						printf("> Unrecognized standard input try 'help'\n");
					}
				}
				else if(!strcmp(standard_input, "list")) {
					if(!strcmp(strptr, "all")) {
						display_module_info_all_long(server_info);
					}
					else if(!strcmp(strptr, "users")) {
						display_user_list(server_info);
					}
					else if(strptr) {
						display_module_info_long_by_name(
								server_info, strptr);
					}
					else {
						printf("> Unrecognized standard input try 'help'\n");
					}
				}
				else if(!strcmp(standard_input, "set")) {
					tmp = strptr;
					strptr = strstr(tmp, " ");
					if(strptr) {
						strptr[0] = 0; strptr ++;
						/* 'tmp' refers to the variable */
						/* 'strptr' refers to the new value to set */
						if(!strcmp(tmp, "number_of_timing_sessions")) {
							server_info->number_of_timing_sessions
									= atoi(strptr);
						}
						else if(!strcmp(tmp, "periodic_time_seconds")) {
							server_info->periodic_time_seconds
									= atoi(strptr);
						}
						else if(!strcmp(tmp, "periodic_time_useconds")) {
							server_info->periodic_time_useconds
									= atoi(strptr);
						}
						else if(!strcmp(tmp, "set_game_in_progress")) {
							server_info->set_game_in_progress
									= atoi(strptr);
						}
						else if(!strcmp(tmp, "set_max_users")) {
							server_info->set_max_users
									= atoi(strptr);
						}
					}
					else {
						printf("> Unrecognized standard input try 'help'\n");
					}
				}
			}
			else {
				printf("> Unrecognized standard input try 'help'\n");
			}
		}
		else {
			printf("> Unrecognized standard input try 'help'\n");
		}
	}

	return 0;
}

/* Find module then unload it & remove its node from the linked list */
int unload_module(struct server_info_hold *server_info, char *module_name) {
	node_module_ptr fix_module_ptr;
	node_module_ptr module_slider;


	/* Find the module node in the linked list */
	module_slider = server_info->module_head;
	if(module_slider == NULL) {
		printf(">! exit 1\n");
		return 1;
	}
	printf(">! comparing '%s' & '%s'\n", module_slider->name, module_name);
	while(strcmp(module_slider->name, module_name)) {
		module_slider = module_slider->next_module;
		if(module_slider == NULL) {
			printf(">! exit 2\n");
			return 1;
		}

		printf(">! comparing '%s' & '%s'\n", module_slider->name, module_name);
	}

	/* Now fix the linked list so we can remove this node */
	fix_module_ptr = server_info->module_head;
	if(fix_module_ptr == module_slider) {
		server_info->module_head = NULL;
	}
	else {
		while(fix_module_ptr->next_module != module_slider) {
			if(fix_module_ptr->next_module == NULL) {
				printf(">! stdin admin tried to unload module not loaded.\n");
				printf(">! exit 3\n");
				return 1;
			}
			fix_module_ptr = fix_module_ptr->next_module;
		}
		fix_module_ptr->next_module = module_slider->next_module;
	} 

	/* Subtract another module to the number of modules connected */
	server_info->number_of_modules_loaded --;

    dlclose(module_slider->handle);
	free(module_slider);

	/* Call modules that need notification of unloading a module */
	module_load_or_unload(server_info);

	return 0;
}

int load_module(struct server_info_hold *server_info, char *module_name) {
	struct initial_info_hold*(*get_info)(void);
	void (*initialize)(struct server_info_hold*);
	void *tmp_handle;
	char buff[512];
	node_module_ptr module_slider;
	node_module_ptr new_module_ptr;

	printf(">! loading module '%s', ", module_name);

	/* Attempt to open the module */
	strcpy(buff, module_name);
	strcat(buff, ".so");
//	printf("'%s': ", buff);
	tmp_handle = dlopen(buff, RTLD_LAZY);
	if(!tmp_handle) {
/*		fputs (dlerror(), stderr);
*/		printf("failed to load.\n");

		return 1;
	}

	strcpy(buff, "initialize");
//	strcpy(buff, "initialize_");
//	strcat(buff, module_name);
//	strcat(buff, "");
	initialize = dlsym(tmp_handle, buff);
	initialize(server_info);

	strcpy(buff, "query_initial_info");
//	strcpy(buff, "query_");
//	strcat(buff, module_name);
//	strcat(buff, "_initial_info");
	get_info = dlsym(tmp_handle, buff);

	/* Allocate a new module node for this new module */
	new_module_ptr = (node_module_ptr) calloc (1, sizeof(node_module_type));

	/* Fill in the new information about the module */
	new_module_ptr->name = strdup(module_name);
	new_module_ptr->handle = tmp_handle;
	new_module_ptr->info = get_info();

	/* Move to the end of the node list & attach */
	module_slider = server_info->module_head;
	if(module_slider == NULL) {
		server_info->module_head = new_module_ptr;
	}
	else if(module_slider->next_module == NULL) {
		module_slider->next_module = new_module_ptr;
	}
	else {
		do {
			module_slider = module_slider->next_module;
		} while(module_slider->next_module != NULL);
		module_slider->next_module = new_module_ptr;
	}

	/* Add another module to the number of modules connected */
	server_info->number_of_modules_loaded ++;

	printf("loaded.\n");

	/* Call modules that need notification of loading a module */
	module_load_or_unload(server_info);

	return 0;
}

void display_help(void) {
	printf("& _Help_\n&\n");
	printf("& Commands available:\n&\n");
	printf("&\t'list users' - lists users connected; id_numbers & nicks\n");
	printf("&\t'slist all' - lists all loaded modules in short form\n");
	printf("&\t'list all' - lists all loaded modules in long form\n");
	printf("&\t'slist <module>' - lists a loaded module in short form\n");
	printf("&\t'list <module>' - lists a loaded module in long form\n");
	printf("& [?]\t'load all' - loads all available modules\n");
	printf("&\t'load <module>' - loads the specified module\n");
	printf("&\t'sload <module>' - loads the specified module silently\n");
	printf("& [?]\t'unload all' - unloads all loaded modules\n");
	printf("&\t'unload <module>' - unloads the specified module\n");
	printf("&\t'quit' - shutdown the server\n");
	printf("&\t'set <variable> <newvalue>' - assign new server variable values\n");
	printf("&\t   set number_of_timing_sessions <integer>\n");
	printf("&\t   set periodic_time_seconds <integer>\n");
	printf("&\t   set periodic_time_useconds <integer>\n");
	printf("&\t   set set_game_in_progress <integer>\n");
	printf("&\t   set set_max_users <integer>\n");
}
