#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/define.h"
#include "../common/server.h"
#include "network.h"
#include "parsestd.h"
#include "parsecli.h"
#include "loadup.h"
#include "execute.h"
#include "periodic.h"

void setup_initial_server_values(struct server_info_hold*);

int main(int argc, char *argv[]) {
	struct server_info_hold server_info;
	int net_status;
	int timing_session_active, ready_for_next_timing_session;
	char *standard_input;
	char *ptr, buff[512];

	/* Set up some intial values */
	setup_initial_server_values(&server_info);

	/* Load up some initial commands from ~/.robot_race/server.com */
	ptr = getenv("HOME");
	strcpy(buff, ptr);
	strcat(buff, "/.robot_race/server.com");
	loadup_command_file(&server_info, buff);

	printf("Robot Race (server v%s)\n", RRSER_VERSION);
	if(argc != 2) {
		printf("Syntax is: 'rrser port_number'\n");
		return 1;
	}
	if(!network_startup(&server_info, atoi(argv[1]))) {
		printf("error: port initialization failed, exiting...\n");
		return 1;
	}
	else {
		printf("Port initialized: listening...\n");
	}

	/* The toggle for the next timing session is always ready in */
	/* anticipation for when the timing session becomes active */
	ready_for_next_timing_session = TRUE;

	for(;;) {
		/* Turn on timing session when the variable number of timing */
		/* sessions becomes greater than zero, logically... */
		timing_session_active = server_info.number_of_timing_sessions;

		/* Launch all modules which have requested to be active during */
		/* this particular timing session. */
		if(timing_session_active) {
			if(ready_for_next_timing_session) {
				ready_for_next_timing_session = FALSE;
				do {
					execute_launch_timing_session(&server_info);
				}	while(
					execute_is_ready_for_next_timing_session(&server_info));
			}
			else {
				/* The timing session has already taken place. Check to see */
				/* if the modules are all ready to move to the next timing */
				/* session. */
				if(execute_is_ready_for_next_timing_session(&server_info)) {
					ready_for_next_timing_session = FALSE;
					do {
						execute_launch_timing_session(&server_info);
					}	while(
						execute_is_ready_for_next_timing_session(&server_info));
				}
			}
		}

		/* See if there is any data available to the server */
		net_status = network_update_buffers(&server_info, &standard_input);

		/* Handle the data available, if there is any */
		switch(net_status) {
			case 0:	periodic_server_call(&server_info);
					printf("* 0 - Initiating periodic free loop: ");
					printf("This is a good place to PING\n");
					break;
			case 1:	standard_input[strlen(standard_input)-1] = 0;
					parse_stdin(&server_info, standard_input);
					free(standard_input);
					break;
			case 2:	printf("* 2 - A new user connected to the network\n");
					break;
			case 3:	printf("* 3 - There are characters in the users buffer\n");
					parse_client(&server_info);
					break;
			case 4:	printf("* 4 - A user has dropped their connection\n");
					break;
			default:	printf("* DEFAULT: This should not be reached.\n");
						printf("* DEFAULT: #%d\n", net_status);
		}
	}

	return 0;
}

void setup_initial_server_values(struct server_info_hold *server_info) {
	unsigned int ctr;

	server_info->user_head = NULL;
	server_info->module_head = NULL;
	server_info->number_of_users_connected = 0;
	server_info->number_of_modules_loaded = 0;
	server_info->number_of_timing_sessions = 0;
	server_info->periodic_time_seconds = 60;
	server_info->periodic_time_useconds = 0;
	server_info->set_max_users = 1;	/* Must be less than MAX_USER_TOGGLES */
	server_info->set_game_in_progress = 0;
	for(ctr = 0; ctr < MAX_USER_TOGGLES; ctr ++) {
		server_info->user_id_toggles[ctr] = FALSE;
	}
}
