#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "start.h"
#include "quit.h"
#include "access.h"
#include "s_admin.h"
#include "module_list_all.h"

/* Global variables related to module description */
struct initial_info_hold initial_info;
unsigned int list_of_network_commands[]
		= { C_ADMIN_START_GAME, C_ADMIN_STOP_GAME, C_ADMIN_GAIN_ACCESS };
struct timing_hold list_timing_sessions[2];
unsigned int list_of_priorities_1[] = { 30000 };
unsigned int list_of_priorities_2[] = { 45000 };
char *multi_description[] = {
	"Takes care of configuring the game as ordered by a client who has sent",
	"the administrative password.  Tells the modules when the game has",
	"began so they can do any necessary preparation/ resetting of status."
	};
char *list_of_styles[] = { "other", "module_load_or_unload" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables related to module function */

/* Initialization function called first right after module is loaded */
void initialize(struct server_info_hold *server_info) {
	/* Set how many players are allowed to play this game. */
	server_info->set_max_users = MAX_USERS;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Timing sessions are not required. */
    initial_info.num_of_timing_sessions = 2;
    list_timing_sessions[0].timing_session = BEFORE_PROGRAM_CARDS_ARE_DELT;
    list_timing_sessions[0].num_of_priorities = 1;
    list_timing_sessions[0].list_of_priorities = list_of_priorities_1;
    list_timing_sessions[1].timing_session = END_OF_TURN;
    list_timing_sessions[1].num_of_priorities = 1;
    list_timing_sessions[1].list_of_priorities = list_of_priorities_2;
    initial_info.list_timing_sessions = list_timing_sessions;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Handles administrative network commands.");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 3;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 3;
	initial_info.list_of_network_commands = list_of_network_commands;

	/* A style is not required */
	initial_info.num_of_styles = 2;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/* Called once before each game */
void init_start_game(server_info)
struct server_info_hold *server_info;
{
}

/* Called once after each game */
void init_quit_game(server_info)
struct server_info_hold *server_info;
{
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

int network_command_handler(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	switch(user_node->buffer[0]) {
		case C_ADMIN_START_GAME:
				/* Remove all the chars involved with this protocol */
				remove_chars_from_user_buffer(user_node, 1);
				case_admin_start_game(server_info);
				break;
		case C_ADMIN_STOP_GAME:
				/* Remove all the chars involved with this protocol */
				remove_chars_from_user_buffer(user_node, 1);
				case_admin_stop_game(server_info);
				break;
		case C_ADMIN_GAIN_ACCESS:
				case_admin_gain_access(server_info, user_node);
				break;
	}

	return 0;
}

/* Handle timing & priorities */
int action_handle_timing(server_info, timing_session, priority)
    struct server_info_hold *server_info;
    unsigned int timing_session;
    unsigned int priority;
{
	printf("[ADMIN] timing %d\n", timing_session);

    switch(timing_session) {
        case BEFORE_PROGRAM_CARDS_ARE_DELT:
				/* Since we are only active during one priority we know this one
				 * is 30000.
				 */

				// I believe this timing session is not necessary and can be
				// removed entirely.
				break;
        case END_OF_TURN:
				/**
				 * Check if the game is no longer in progress
				 */
				if(server_info->set_game_in_progress == 2) {
					case_admin_stop_game(server_info);
				}
				break;
	}

	return 0;
}                                                                                                            
int module_load_or_unload(server_info)
struct server_info_hold *server_info;
{
	/* assign id values to each of the loaded modules, these are */
	/* used at various times for many things but mostly for the */
	/* S_ACTION command... loading and unloading modules will */
	/* screw this up so when a module is loaded or unloaded these */
	/* values are reassigned */
	assign_id_values(server_info);

	/* send the new listing to all the users connected */
	send_module_list_to_all(server_info);

	return 1;
}
