/**
 * User wishes to change their nick, all users will be informed of the change.
 *
 * Author: Travis W. Loyd
 */

#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "userlist.h"
#include "addrobot.h"
#include "module_list.h"

/* Implements server response to C_CHANGE_NICK */
/* ## - length of the new nick including null */
/* SS - the string of the new nick */
void case_change_nick(server_info, user_slider)
struct server_info_hold *server_info;
struct node_user_type *user_slider;
{
	int length_of_nick;
	char buff[512];
	char tosend[512];

	length_of_nick = user_slider->buffer[1];
	strncpy(buff, user_slider->buffer+2, length_of_nick);

	/* Is this the user's first nick? Then they joined! */
	if(user_slider->nick == NULL) {
		user_slider->nick = strdup(buff);
		
		/* This user is new to this server, send the module listing. */
		send_module_list(server_info, user_slider);

		/* Create a robot node for this user, if no game yet */
		/* The robot node will be removed once the game starts if the */
		/* user decides they do not wish to participate in the game. */
		if(!server_info->set_game_in_progress) {
			add_robot_node(server_info, user_slider);
		}

		/* Send network S_JOIN command to all clients */
		tosend[0] = S_JOIN;
		tosend[1] = user_slider->id_number;
		tosend[2] = strlen(user_slider->nick) + 1;
		strcpy(tosend + 3, user_slider->nick);
		network_say_to_all_but(
				server_info, tosend
				, tosend[2] + 3
				, user_slider->id_number);

		/* Send network S_USERLIST command to client */
		send_userlist(server_info, user_slider);
	}
	else {
		/* Free nick memory before allocating new memory */
		free(user_slider->nick);
		user_slider->nick = strdup(buff);

		/* Send network S_CHANGE_NICK command to client */
		tosend[0] = S_CHANGE_NICK;
		tosend[1] = user_slider->id_number;
		tosend[2] = strlen(user_slider->nick) + 1;
		strcpy(tosend + 3, user_slider->nick);
		network_say_to_all(
				server_info, tosend
				, tosend[2] + 3);
	}

	/* Remove all the chars involved with this protocol */
	remove_chars_from_user_buffer(user_slider, length_of_nick + 2);
}
