/**
 * Handles basic communication within the main channel. Special functions
 * are called if the user has just joined the server and sent in his/her
 * nick, see: 'nick.c'
 *
 * Module Dependencies: {robot, see: 'addrobot.*'}
 * Author: Travis W. Loyd
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "nick.h"
#include "talk.h"
#include "emote.h"
#include "whisper.h"

/* Variables global to the module */
struct initial_info_hold initial_info;
unsigned int list_of_network_commands[]
		= { C_CHANGE_NICK, C_TALK, C_EMOTE, C_WHISPER };
char *multi_description[] = { "this is line 1", "this is line 2" };
char *list_of_styles[] = { "other", "user_dropped_their_connection" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "1.0" };

/* Initialization function called first right after module is loaded */
void initialize(void) {
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Timings are not required */
	initial_info.num_of_timing_sessions = 0;
	initial_info.list_timing_sessions = NULL;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Answers & produces network protocol for basic chat");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 2;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 4;
	initial_info.list_of_network_commands = list_of_network_commands;

	/* A style is not required */
	initial_info.num_of_styles = 2;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

/* Called once before each game */
void init_start_game(server_info)
struct server_info_hold *server_info;
{
}

/* Called once after each game */
void init_quit_game(server_info)
struct server_info_hold *server_info;
{
}

int network_command_handler(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	switch(user_node->buffer[0]) {
		case C_CHANGE_NICK:
				case_change_nick(server_info, user_node);
				break;
		case C_TALK:
				case_talk(server_info, user_node);
				break;
		case C_EMOTE:
				case_emote(server_info, user_node);
				break;
		case C_WHISPER:
				case_whisper(server_info, user_node);
				break;
	}

	return 0;
}

int network_user_dropped_connection(server_info, id_number)
struct server_info_hold *server_info;
int id_number;
{
	char tosend[2];

	tosend[0] = S_QUIT;
	tosend[1] = id_number;
	network_say_to_all(server_info, tosend, 2);
			 
    return 0;
}                                                                                            
