/**
 *
 * Author: Travis W. Loyd
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "addnode.h"
#include "delnode.h"
#include "action.h"

/* Global variables related to module description */
struct initial_info_hold initial_info;
struct timing_hold list_timing_sessions[5];
unsigned int list_of_priorities_1[] = { 37500 };
char *multi_description[] = {
	"Implements the checkpoint element of the basic game.",
	"If a moveable object is standing on a checkpoint when it is",
	"activated the object will be rotated either clockwise",
	"or counterclockwise depending on the checkpoint."
	};
char *list_of_styles[] = { "element" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables related to module function */
unsigned int number_of_basic_checkpoints;
struct node_checkpoint_type *checkpoint_head;
int last_checkpoint;

/* Initialization function called first right after module is loaded */
void initialize(struct server_info_hold *server_info) {
	number_of_basic_checkpoints = 0;
	checkpoint_head = NULL;
	last_checkpoint = 0;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* No timing sessions in this module */
	initial_info.num_of_timing_sessions = 5;
	list_timing_sessions[0].timing_session = FIRST_PHASE;
	list_timing_sessions[0].num_of_priorities = 1;
	list_timing_sessions[0].list_of_priorities = list_of_priorities_1;
	list_timing_sessions[1].timing_session = SECOND_PHASE;
	list_timing_sessions[1].num_of_priorities = 1;
	list_timing_sessions[1].list_of_priorities = list_of_priorities_1;
	list_timing_sessions[2].timing_session = THIRD_PHASE;
	list_timing_sessions[2].num_of_priorities = 1;
	list_timing_sessions[2].list_of_priorities = list_of_priorities_1;
	list_timing_sessions[3].timing_session = FOURTH_PHASE;
	list_timing_sessions[3].num_of_priorities = 1;
	list_timing_sessions[3].list_of_priorities = list_of_priorities_1;
	list_timing_sessions[4].timing_session = FIFTH_PHASE;
	list_timing_sessions[4].num_of_priorities = 1;
	list_timing_sessions[4].list_of_priorities = list_of_priorities_1;
	initial_info.list_timing_sessions = list_timing_sessions;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Implements the checkpoint element of the basic game.");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 4;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 0;
	initial_info.list_of_network_commands = NULL;

	/* A style is not required */
	initial_info.num_of_styles = 1;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

int add(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	int result;

	last_checkpoint++;

	printf("Adding Gear to (%d,%d)x(%d,%d) spin: %d\n",
		user_node->buffer[0],
		user_node->buffer[1],
		user_node->buffer[2],
		user_node->buffer[3],
		user_node->buffer[4]
		);
	printf("Translation: %ld, %ld\n",
		(long)(user_node->buffer[0] * 250 + user_node->buffer[1]),
		(long)(user_node->buffer[2] * 250 + user_node->buffer[3])
		);

	result = add_checkpoint_node(server_info, &checkpoint_head, user_node);
	number_of_basic_checkpoints++;

	/* Remove characters involved from the network socket */
	remove_chars_from_user_buffer(user_node, 5);

	/* Return whether or not the element was added successfully */
	return result;
}

int del(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	return 1;
}

void init_start_game(server_info)
struct server_info_hold *server_info;
{
	node_module_ptr module_slider;
	node_checkpoint_ptr checkpoint_slider;
	char tosend[7];
	int module_id;

	printf("[BASIC_CHECKPOINT] Initializing module...\n");

	printf("[BASIC_CHECKPOINT] Sending element placement.\n");

	/* first, which reference number is this checkpoint module? */
	module_id = 0;
	module_slider = server_info->module_head;
	while(module_slider) {
		if(!strcmp(module_slider->name, "s_basic_checkpoint")) {
			break;
		}
		module_id++;
		module_slider = module_slider->next_module;
	}
	if(!module_slider) {
		printf("[DEBUG] (basic_checkpoint) This point should never be reached.\n");
		return;
	}

	/* cycle through linked list displaying element placements */
	checkpoint_slider = checkpoint_head;
	while(checkpoint_slider) {
		tosend[0] = S_ELEMENT_PLACEMENT;
		tosend[1] = module_id;
		tosend[2] = checkpoint_slider->x / 250;
		tosend[3] = checkpoint_slider->x % 250;
		tosend[4] = checkpoint_slider->y / 250;
		tosend[5] = checkpoint_slider->y % 250;
		tosend[6] = checkpoint_slider->type;

		network_say_to_all(server_info, tosend, 7);

		/* move to next element */
		checkpoint_slider = checkpoint_slider->next_checkpoint;
	}
//	printf("done.\n");
}

void init_quit_game(server_info)
struct server_info_hold *server_info;
{
	printf("[BASIC_CHECKPOINT] removing all checkpoint nodes.\n");
	del_all_checkpoint_nodes(server_info, &checkpoint_head);
}

void action_handle_timing(server_info, timing_session, priority)
struct server_info_hold *server_info;
unsigned int timing_session;
unsigned int priority;
{
	printf("[BASIC_CHECKPOINT] Action: Timing=%d, Priority=%d\n"
		, timing_session
		, priority);

	if(last_checkpoint != -1) {
		tag_checkpoints(server_info, checkpoint_head, last_checkpoint);
	}
	else {
		printf("Last checkpoint has been tagged, game should be over.\n");
	}
}
