/**
 *
 * Author: Travis W. Loyd
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "addnode.h"
#include "delnode.h"

/* Global variables related to module description */
struct initial_info_hold initial_info;
char *multi_description[] = {
	"Implements the wall element of the basic game.",
	"If a moveable object is standing on a wall when it is",
	"activated the object will be rotated either clockwise",
	"or counterclockwise depending on the wall."
	};
char *list_of_styles[] = { "element", "stop" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables related to module function */
unsigned int number_of_basic_walls;
struct node_wall_type *wall_head;

/* Initialization function called first right after module is loaded */
void initialize(struct server_info_hold *server_info) {
	number_of_basic_walls = 0;
	wall_head = NULL;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* No timing sessions in this module */
    initial_info.num_of_timing_sessions = 0;
    initial_info.list_timing_sessions = NULL;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Implements the wall element of the basic game.");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 4;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 0;
	initial_info.list_of_network_commands = NULL;

	/* A style is not required */
	initial_info.num_of_styles = 2;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

int add(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	int result;

	printf("Adding Wall to (%d,%d)x(%d,%d) instance: %d\n",
		user_node->buffer[0],
		user_node->buffer[1],
		user_node->buffer[2],
		user_node->buffer[3],
		user_node->buffer[4]
		);
	printf("Translation: %ld, %ld\n",
		(long)(user_node->buffer[0] * 250 + user_node->buffer[1]),
		(long)(user_node->buffer[2] * 250 + user_node->buffer[3])
		);

	result = add_wall_node(server_info, &wall_head, user_node);
	number_of_basic_walls++;

	/* Remove characters involved from the network socket */
	remove_chars_from_user_buffer(user_node, 5);

	/* Return whether or not the element was added successfully */
	return result;
}

int del(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	return 1;
}

void init_start_game(server_info)
struct server_info_hold *server_info;
{
	node_module_ptr module_slider;
	node_wall_ptr wall_slider;
	char tosend[7];
	int module_id;

	printf("[BASIC_WALL] Initializing module...\n");

	printf("[BASIC_WALL] Sending element placement.\n");

	/* first, which reference number is this wall module? */
	module_id = 0;
	module_slider = server_info->module_head;
	while(module_slider) {
		if(!strcmp(module_slider->name, "s_basic_wall")) {
			break;
		}
		module_id++;
		module_slider = module_slider->next_module;
	}
	if(!module_slider) {
		printf("[DEBUG] (basic_wall) This point should never be reached.\n");
		return;
	}

	/* cycle through linked list displaying element placements */
	wall_slider = wall_head;
	while(wall_slider) {
		tosend[0] = S_ELEMENT_PLACEMENT;
		tosend[1] = module_id;
		tosend[2] = wall_slider->x / 250;
		tosend[3] = wall_slider->x % 250;
		tosend[4] = wall_slider->y / 250;
		tosend[5] = wall_slider->y % 250;
		tosend[6] = wall_slider->type;

		network_say_to_all(server_info, tosend, 7);

		/* move to next element */
		wall_slider = wall_slider->next_wall;
	}
//	printf("done.\n");
}

void init_quit_game(server_info)
struct server_info_hold *server_info;
{
	printf("[BASIC_WALL] removing all wall nodes.\n");
	del_all_wall_nodes(server_info, &wall_head);
}

/**
 * This module has specified that it has the style 'stop' which
 * means that it can stop moveable objects. This module of basic_wall
 * will ALWAYS stop ANY moveable objects.
 */
int check4stop(server_info, from_x, from_y, to_x, to_y)
struct server_info_hold *server_info;
long from_x;
long from_y;
long to_x;
long to_y;
{
	struct node_wall_type *wall_slider;
	int in_front;
	int in_back;

	/**
	 * We will search for two walls.
	 *
	 * 1. Is there a wall on the square we are in, in the direction
	 *    we are going?
	 * 2. Is there a wall in the next square where we want to go?
	 */

	/* What side of the square is in front of this object? */
	if(to_y < from_y) {
		in_front = 0;
		in_back = 2;
	}
	else if(to_x > from_x) {
		in_front = 1;
		in_back = 3;
	}
	else if(to_y > from_y) {
		in_front = 2;
		in_back = 0;
	}
	else if(to_x < from_x) {
		in_front = 3;
		in_back = 1;
	}

	/* search for a possible wall in this square */
	wall_slider = wall_head;
	while(wall_slider) {
		/* 1. */
		if(wall_slider->x == from_x) {
			if(wall_slider->y == from_y) {
				if(wall_slider->type == in_front) {
					return TRUE;
				}
			}
		}
		/* 2. */
		if(wall_slider->x == to_x) {
			if(wall_slider->y == to_y) {
				if(wall_slider->type == in_back) {
					return TRUE;
				}
			}
		}

		/* check the next wall */
		wall_slider = wall_slider->next_wall;
	}

	return FALSE;
}
