/**
 *
 *
 * Author: Travis W. Loyd
 */

#include <stdio.h>
#include <dlfcn.h>
#include "../common/loadall.h"
#include "s_p_card.h"
#include "instant.h"
#include "is_shutdown.h"

void send_result(struct server_info_hold*, int, int, int, int, int);

void reveal_all_program_cards(server_info, p_card_head, phase)
struct server_info_hold *server_info;
struct node_p_card_type **p_card_head;
int phase;
{
	int (*_move_with_check)(struct server_info_hold*, int, int);
	struct node_module_type *module_slider;
	struct node_p_card_type *p_card_slider;
	int curr_pcard;
	int move_type;
	int result;
	int robot_ref_id;
	int p_card_ref_id;

	printf("[P_CARD] revealing next program card in phase: %d\n", phase);

	/**
	 * find reference id numbers for network communications
	 */
	module_slider = server_info->module_head;
	while(module_slider) {
		if(!strcmp(module_slider->name, "s_p_card")) {
			p_card_ref_id = module_slider->id;
			break;
		}

		module_slider = module_slider->next_module;
	}
	if(!module_slider) {
		printf("[P_CARD] this point should never be reached 1\n");
		return;
	}

	module_slider = server_info->module_head;
	while(module_slider) {
		if(!strcmp(module_slider->name, "s_robot")) {
			robot_ref_id = module_slider->id;
			break;
		}

		module_slider = module_slider->next_module;
	}
	if(!module_slider) {
		printf("[P_CARD] this point should never be reached 2\n");
		return;
	}

	_move_with_check = dlsym(module_slider->handle,
			"move_with_check");

	p_card_slider = (*p_card_head);
	while(p_card_slider) {
		if(!is_shutdown(server_info, p_card_slider->roboNum)) {
			printf("[P_CARD] Robot #%ld, has card: %d\n",
					p_card_slider->roboNum,
					p_card_slider->selected[phase]);

			/**
			 * find movement type according to program card
			 * and then call the movement
			 */
			curr_pcard = p_card_slider->selected[phase];
			if(curr_pcard <= 6) {
				/* u_turn */
				printf("performing u_turn\n");
				move_type = U_TURN;
				result = _move_with_check(server_info, U_TURN, p_card_slider->roboNum);
				send_result(server_info, U_TURN,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
/* now check if anything was killed and send network chat about it */
			}
			else if(curr_pcard <= 42) {
				if(!(curr_pcard % 2)) {
					/* rotate_right */
					printf("performing rotate_right\n");
					move_type = ROTATE_RIGHT;
					result = _move_with_check(server_info, ROTATE_RIGHT, p_card_slider->roboNum);
					send_result(server_info, ROTATE_RIGHT,
							result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
					activate_all_instant(server_info);
				}
				else {
					/* rotate_left */
					printf("performing rotate_left\n");
					move_type = ROTATE_LEFT;
					result = _move_with_check(server_info, ROTATE_LEFT, p_card_slider->roboNum);
					send_result(server_info, ROTATE_LEFT,
							result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
					activate_all_instant(server_info);
				}
			}
			else if(curr_pcard <= 48) {
				/* back_up */
				printf("performing back_up\n");
				move_type = BACK_UP;
				result = _move_with_check(server_info, BACK_UP, p_card_slider->roboNum);
				send_result(server_info, BACK_UP,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
			}
			else if(curr_pcard <= 66) {
				/* forward_1 */
				printf("performing forward_1\n");
				move_type = FORWARD;
				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
			}
			else if(curr_pcard <= 78) {
				/* forward_2 */
				printf("performing forward_2\n");
				move_type = FORWARD;
				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
	
				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
			}
			else {
				/* forward_3 */
				printf("performing forward_3\n");
				move_type = FORWARD;
				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
	
				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);

				result = _move_with_check(server_info, FORWARD, p_card_slider->roboNum);
				send_result(server_info, FORWARD,
						result,p_card_slider->roboNum, p_card_ref_id, robot_ref_id);
				activate_all_instant(server_info);
			}
		}

		p_card_slider = p_card_slider->next_p_card;
	}
}

void send_result(server_info, movement, worked, roboNum, p_card_ref_id, robot_ref_id)
struct server_info_hold *server_info;
int movement;
int worked;
int roboNum;
int p_card_ref_id;
int robot_ref_id;
{
	char tosend[7];

	tosend[0] = S_ACTION;
	tosend[1] = p_card_ref_id;
	tosend[2] = -1;
	tosend[3] = robot_ref_id;
	tosend[4] = roboNum;
	tosend[5] = 0;

	if(worked != -1) {
		/* movement was stopped */
		tosend[6] = movement + 10;
	}
	else {
		/* movement occurred */
		tosend[6] = movement;
	}

	network_say_to_all(server_info, tosend, 7);
}
