#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include "../common/loadall.h"
#include "dealpcard.h"
#include "listing.h"
#include "shuffle.h"
#include "s_p_card.h"
#include "addnode.h"
#include "delnode.h"
#include "reveal.h"

/* Global variables related to the module description */
struct initial_info_hold initial_info;
unsigned int list_of_network_commands[]
        = { C_P_CARD_SELECTION, C_I_WANT_TO_SHUTDOWN,
        	FIRST_PHASE, SECOND_PHASE, THIRD_PHASE,
			FOURTH_PHASE, FIFTH_PHASE };
struct timing_hold list_timing_sessions[7];
unsigned int list_of_priorities_1[] = { 45000 };
unsigned int list_of_priorities_2[] = { 1 };
unsigned int list_of_priorities_3[] = { 3750 };
char *multi_description[] = {
	"Shuffle & deal locally program cards.  Later timing sessions",
	"cause the program cards to be revealed and call movement in",
	"the robot module.  Tries to create enough functions so that",
	"option cards can manipulate the program cards easily."
	};
char *list_of_styles[] = { "other", "user_dropped_their_connection" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.1" };

/* Global variables related to the module function */
unsigned int g_the_deck[84];
int g_next_program_card;
struct node_p_card_type *p_card_head;

/* Initialization function called first right after module is loaded */
void initialize(struct server_info_hold *server_info) {
	unsigned int ctr;

	/* Fill the deck with cards */
	for(ctr = 0; ctr < NUM_PCARDS; ctr ++) {
		g_the_deck[ctr] = (ctr + 1); /* * 10; */
	}

	/* Wait till all 5 program cards are played before continuing. */
	g_next_program_card = 0;

	/* set head of link list to null */
	p_card_head = NULL;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
    /* Adding some timings here just for testing purposes */
    initial_info.num_of_timing_sessions = 7;
    list_timing_sessions[0].timing_session = BEFORE_PROGRAM_CARDS_ARE_DELT;
    list_timing_sessions[0].num_of_priorities = 1;
    list_timing_sessions[0].list_of_priorities = list_of_priorities_1;
    list_timing_sessions[1].timing_session = AFTER_PROGRAM_CARDS_ARE_DELT;
    list_timing_sessions[1].num_of_priorities = 1;
    list_timing_sessions[1].list_of_priorities = list_of_priorities_2;
    list_timing_sessions[2].timing_session = FIRST_PHASE;
    list_timing_sessions[2].num_of_priorities = 1;
    list_timing_sessions[2].list_of_priorities = list_of_priorities_3;
    list_timing_sessions[3].timing_session = SECOND_PHASE;
    list_timing_sessions[3].num_of_priorities = 1;
    list_timing_sessions[3].list_of_priorities = list_of_priorities_3;
    list_timing_sessions[4].timing_session = THIRD_PHASE;
    list_timing_sessions[4].num_of_priorities = 1;
    list_timing_sessions[4].list_of_priorities = list_of_priorities_3;
    list_timing_sessions[5].timing_session = FOURTH_PHASE;
    list_timing_sessions[5].num_of_priorities = 1;
    list_timing_sessions[5].list_of_priorities = list_of_priorities_3;
    list_timing_sessions[6].timing_session = FIFTH_PHASE;
    list_timing_sessions[6].num_of_priorities = 1;
    list_timing_sessions[6].list_of_priorities = list_of_priorities_3;
    initial_info.list_timing_sessions = list_timing_sessions;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Shuffle, deal, and remember which robots have which cards.");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 4;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 2;
	initial_info.list_of_network_commands = list_of_network_commands;

	/* A style is not required */
	initial_info.num_of_styles = 2;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
	struct node_p_card_type* p_card_slider;

	p_card_slider = p_card_head;
	while(p_card_slider) {
		if(!p_card_slider->is_ready) {
			return FALSE;
		}
		p_card_slider = p_card_slider->next_p_card;
	}
    return TRUE;
}

/* Called once before each game */
void init_start_game(server_info)
struct server_info_hold *server_info;
{
	int (*is_player)(struct server_info_hold*, int);
	struct node_module_type* module_slider;
	struct node_user_type* user_slider;

	printf("[P_CARD] Initializing module...\n");
			 
	/**
	 * For each user, if they are a player create a
	 * program card node and add it to the link list.
	 */

	/* Find the robot module we need */
	module_slider = server_info->module_head;
	while(module_slider) {
		if(!strcmp(module_slider->name, "s_robot")) {
			break;
		}
		module_slider = module_slider->next_module;
	}
	is_player = dlsym(module_slider->handle, "is_player");                            

	/* add a program card node to represent each player */
	user_slider = server_info->user_head;
	while(user_slider) {
		if(is_player(server_info, user_slider->id_number)) {
			add_p_card_node(server_info, &p_card_head, user_slider);
		}

		user_slider = user_slider->next_user;
	}
}

void init_quit_game(server_info)
struct server_info_hold *server_info;
{
	struct node_p_card_type *p_card_slider;
	long kill_me;

	printf("[P_CARD] QUITTING: removing p_card nodes\n");
	printf("[P_CARD] removing: ");

	p_card_slider = p_card_head;
	while(p_card_slider) {
		kill_me = p_card_slider->roboNum;
		p_card_slider = p_card_slider->next_p_card;

		printf("[%ld", kill_me);
    	del_p_card_node(
				server_info,
				&p_card_head,
				kill_me);
		printf("(ok)] ");
	}
	printf("\n");
}

/* Initialization function called first right after module is loaded */
void action_handle_timing(server_info, timing_session, priority)
struct server_info_hold *server_info;
unsigned int timing_session;
unsigned int priority;
{

    printf("[P_CARD] Action: Timing=%d, Priority=%d\n"
			, timing_session
			, priority);

    switch(timing_session) {
    	case AFTER_PROGRAM_CARDS_ARE_DELT:
			/* deal program cards to all the robots */
			printf("[P_CARD] dealing program cards to all players\n");
			case_deal_program_cards(server_info, &p_card_head, g_the_deck);
			break;
    	case BEFORE_PROGRAM_CARDS_ARE_DELT:
			/* shuffle the deck */
			printf("[P_CARD] shuffling deck\n");
			shuffle_deck(g_the_deck);
			break;
		default:
			/* by default we are revealing cards */
			reveal_all_program_cards(
					server_info, &p_card_head, timing_session - FIRST_PHASE);
	}
}                       

int network_command_handler(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	switch(user_node->buffer[0]) {
		case C_P_CARD_SELECTION:
			case_program_card_list(server_info, &p_card_head, user_node);
			break;
	}

	return 0;
}

int network_user_dropped_connection(server_info, id_number)
struct server_info_hold *server_info;
int id_number;
{
	printf("[DEBUG] (p_card): We see that %d has dropped connection.\n",
		id_number);

	/* removed the program card node representing this user if there */
	/* is one */
	del_p_card_node(server_info, &p_card_head, id_number);

	return 0;
}
