/**
 * Loop through robot link list removing the nodes which the user
 * has decided not to play.
 *
 * Author: Travis W. Loyd
 */

#include <stdlib.h>
#include "../common/loadall.h"

int remove_nonplayers_from_robot_link_list(server_info, robot_head)
struct server_info_hold *server_info;
struct node_robot_type **robot_head;
{
	node_robot_ptr robot_slider;
	node_robot_ptr tmp_ptr;

	/* While removing the first node... */
	robot_slider = (*robot_head);
	while(robot_slider) {
		/* Removing the head node is pretty easy */
		if(!robot_slider->will_play_when_game_starts) {
			(*robot_head) = robot_slider->next_robot;
			free(robot_slider);

			/* Now we will check the new head node */
			robot_slider = (*robot_head);
		}
		else {
			break;
		}
	}

	/* We may be out of robot nodes, if so just exit the function */
	if(!robot_slider) {
		return 0;
	}
	
	/* Now look for either middle nodes to remove or the end node */
	while(robot_slider->next_robot) {
		if(!robot_slider->next_robot->will_play_when_game_starts) {
			/* Remove this 'in-the-middle' node */

			/* Keep track of the node we are removing */
			tmp_ptr = robot_slider->next_robot;

			/* Attach the nodes in the link list back together */
			robot_slider->next_robot = robot_slider->next_robot->next_robot;

			/* Free up the memory used by the node being removed */
			free(tmp_ptr);
		}
		else {
			/* Now we will check the next robot node */
			robot_slider = robot_slider->next_robot;
		}
	}

	return 1;
}
