#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common/loadall.h"
#include "loadmod.h"
#include "add.h"
#include "del.h"

/* Global variables related to module description */
struct initial_info_hold initial_info;
unsigned int list_of_network_commands[]
		= { C_ADMIN_ADD_ELEMENT, C_ADMIN_DEL_ELEMENT,
			C_ADMIN_SET_WORLD_WIDTH, C_ADMIN_SET_WORLD_HEIGHT,
	   		C_ADMIN_SET_NUMBER_OF_ELEMENTS };
char *multi_description[] = {
	"Interface to adding and removing elements from the world.",
	"Also holds two 'long's which contain the size of the world."
	};
char *list_of_styles[] = { "other" };
char author_of_module[] = { "Travis W. Loyd" };
char version_of_module[] = { "0.01" };

/* Global variables related to module function */
int width;
int height;
long number_of_elements;

/* Initialization function called first right after module is loaded */
void initialize(struct server_info_hold *server_info) {
	width = height = number_of_elements = 0;
}

/* Initial query by game engine to get module information, called second */
struct initial_info_hold* query_initial_info(void) {
	/* Timing sessions are not required. */
    initial_info.num_of_timing_sessions = 0;
    initial_info.list_timing_sessions = NULL;

	/* Single line descriptions are required */
	initial_info.one_line_description = strdup(
		"Interface to adding and removing elements from the world.");

	/* Multi-line descriptions are not required */
	initial_info.num_of_lines_in_multi_description = 2;
	initial_info.multi_description = multi_description;

	/* Network command handling is not required */
	initial_info.num_of_network_commands = 5;
	initial_info.list_of_network_commands = list_of_network_commands;

	/* A style is not required */
	initial_info.num_of_styles = 1;
	initial_info.list_of_styles = list_of_styles;

	/* Set author & version */
	initial_info.author = author_of_module;
	initial_info.version = version_of_module;

	return &initial_info;
}

/* Called once before each game */
void init_start_game(server_info)
struct server_info_hold *server_info;
{
	char tosend[512];
	char tmp[512];

	tosend[0] = S_WORLD_SIZE_IS;
	tosend[1] = width;
	tosend[2] = height;
	network_say_to_all(server_info, tosend, 3);

	sprintf(tmp, "%ld", number_of_elements);
	tosend[0] = S_NUMBER_OF_ELEMENTS;
	tosend[1] = strlen(tmp)+1;
	strcpy(tosend+2, tmp);
	network_say_to_all(server_info, tosend, 3+strlen(tmp));

	printf("sending this many characters: %d\n", 3+strlen(tmp));
	printf("there are %ld elements\n", number_of_elements);
}

void init_quit_game(server_info)
struct server_info_hold *server_info;
{
	number_of_elements = 0;
}

/**
 * This function is required in every module, this module never delays
 * a timeing session so it simply returns TRUE indicating that it is always
 * ready to proceed to the next timeing session.
 */
int is_ready(server_info)
struct server_info_hold *server_info;
{
    return TRUE;
}

int network_command_handler(server_info, user_node)
struct server_info_hold *server_info;
struct node_user_type *user_node;
{
	char buff[512];

	switch(user_node->buffer[0]) {
		case C_ADMIN_ADD_ELEMENT:
				number_of_elements++;
				case_add_element(server_info, user_node);
				break;
		case C_ADMIN_DEL_ELEMENT:
				number_of_elements--;
				break;
		case C_ADMIN_SET_WORLD_WIDTH:
			    /* Remove the characters related to this command */
				width = user_node->buffer[1];
			    remove_chars_from_user_buffer(user_node, 2);
				break;
		case C_ADMIN_SET_WORLD_HEIGHT:
			    /* Remove the characters related to this command */
				height = user_node->buffer[1];
			    remove_chars_from_user_buffer(user_node, 2);
				break;
		case C_ADMIN_SET_NUMBER_OF_ELEMENTS:
				/* set the number of elements in the board */
				strncpy(buff, user_node->buffer+1, user_node->buffer[0]);
			    remove_chars_from_user_buffer(
						user_node, 1+user_node->buffer[0]);
				number_of_elements = atol(buff);
				break;
	}

	return 0;
}

long get_width(void) {
	return width;
}

long get_height(void) {
	return height;
}
